//     Full Function UART        
#define FFRBR      ( 0x40100000 )          // Receive Buffer Register (read only)     
#define FFTHR      ( 0x40100000 )          // Transmit Holding Register (write only)     
#define FFIER      ( 0x40100004 )          // Interrupt Enable Register (read/write)     
#define FFIIR      ( 0x40100008 )          // Interrupt ID Register (read only)     
#define FFFCR      ( 0x40100008 )          // FIFO Control Register (write only)     
#define FFLCR      ( 0x4010000C )          // Line Control Register (read/write)     
#define FFMCR      ( 0x40100010 )          // Modem Control Register (read/write)     
#define FFLSR      ( 0x40100014 )          // Line Status Register (read only)     
#define FFMSR      ( 0x40100018 )          // Modem Status Register (read only)     
#define FFSPR      ( 0x4010001C )          // Scratch Pad Register (read/write)     
#define FFDLL      ( 0x40100000 )          // baud divisor lower byte (read/write)     
#define FFDLH      ( 0x40100004 )          // baud divisor higher byte (read/write)     
#define FFISR      ( 0x40100020 )          // slow Infrared Select Register (read/write)     


#define ANY_DISP_SEG 0x8000      // Set if should be displayed as segmented

#define FALSE 0
#define TRUE 1

typedef unsigned long DWORD;
typedef unsigned short WORD;
typedef unsigned char  BYTE;

typedef int BOOL;


typedef void 	*   PVOID;
typedef DWORD 	*   PDWORD;
typedef WORD 	*   PWORD;
typedef BYTE 	*   PBYTE;
typedef BOOL 	*   PBOOL;
typedef char 	*   PSTR;
typedef const char *    PCSTR;

typedef WORD     ROMWORD;
typedef DWORD    ROMDWORD;
typedef BYTE     ROMBYTE;
typedef char     ROMCHAR;
typedef PCSTR    PROMCHAR;


typedef void (* WRITECHAR)(BYTE);



//
// Address structure for printf %la and ScanAddress:
//
typedef struct {
    WORD     Flags;
    WORD     MemSpace;
    DWORD    LinearAddress;
    PVOID    SegAddress;
} AnyAddress, * FPANYADDRESS;


int
tolower( int value) {
	if (value >= 'A' && value <='Z') 
		return (value + 0x20);
	else
		return value;
}



static void iprintf(char * format, ...);    // Internal printf

static WRITECHAR writechar;

/*
** define global storage for the sprint routine
*/
char sprintbuf[1024];
unsigned short sbindex;




static void output_char(unsigned char data) {
	while(!((*(volatile unsigned long int *)FFLSR) & 0x40)); // wait here for fifo not full
	*(volatile unsigned long int *)FFTHR = (unsigned long int) data;
}




/////////////////////////////////////////////////////////////////////////
// FillSpaces() prints out a given number of spaces or zeroes
//
static void FillSpaces(int Size, BOOL DoIt, BOOL FillZero)
{
    if (DoIt)
        while (Size-- >0)
            writechar((char) (FillZero ? '0' : ' '));
}

/////////////////////////////////////////////////////////////////////////
// HexDigit() returns the correct hexadecimal digit for a number
//
char HexDigit(DWORD value, char UpperLower)
{
    if (value < 10)
        return (char)(value + '0');

    return (char)(value-10 + 'A' + UpperLower - 'X');
}

/////////////////////////////////////////////////////////////////////////
// PrintAddress prints a standard (segment:offset) or long address.
// It is an internal routine used by printf.
//
static void PrintAddress(PDWORD param, BOOL Long, WORD StringIndex)
{
    static BYTE string0[] = "%04X:%04lX";
    static BYTE string1[] = "%04x:%04lx";
    static BYTE string2[] = "%01X/%07lX";
    static BYTE string3[] = "%01x/%07lx";

    static BYTE * strings[] = {string0,string1,string2,string3};

    FPANYADDRESS a;

    DWORD LinearAddress = param[0];
    DWORD  MemSpace      = param[1];
    WORD  SegValue;

    if (Long)
    {
        a = (FPANYADDRESS)*((PDWORD)param);
        LinearAddress = a->LinearAddress;
        MemSpace      = a->MemSpace;
        SegValue      = 0;

        if ((a->Flags & ANY_DISP_SEG) == 0)
            StringIndex += 2;
        else
        {
            MemSpace = ((WORD)(LinearAddress >> 4) - SegValue);
            MemSpace &= 0xF000;
            MemSpace +=  SegValue;
            LinearAddress -= ((DWORD)MemSpace << 4);
        }
    }
    iprintf(strings[StringIndex],MemSpace,LinearAddress);
}

/////////////////////////////////////////////////////////////////////////
// Minimal printf implementation.  Supports:
//  % [-] [0] [<length>] [l] [s|c|x|i|d|u]
//
//  -     == Left justify
//  0     == Fill field with zeroes
// length == Minimum field width
//  l     == Doubleword for x|i|d|u
//  s     == string pointer
//  c     == single character
//  x     == hex number
//  i,d   == signed decimal number
//  u     == unsigned number
//  a     == address (segment:offset)
//  la    == long address (AnyAddress type)
//
void vpprintf(WRITECHAR outroutine, PCSTR * ptrptr)
{
    static BYTE Percent[]      = "%";
    static BYTE PercentQuery[] = "%???";
    static BYTE NullMsg[]      = "";

    PCSTR ptr     = *ptrptr;
    PDWORD param   = (PDWORD)(ptrptr+1);
    BOOL ZeroFill;
    int  MinWidth;
    BOOL Long;
    enum {ParseIncomplete, ParseString, ParseNum, ParsePercent,
                   ParseAddress, ParseNonsense} ParseType;
    BOOL LeftJustify;
    PSTR sPtr;
    WORD NumBase;
    BOOL Signed;
    char ch;
    char NumString[16];          // Holds largest number

    writechar = outroutine;

    for (;;)
    {
        switch (ch = *(ptr++))
        {
            case 0:
               return;
            case '%':
                ZeroFill = FALSE;
                MinWidth = 0;
                Long = FALSE;
                LeftJustify = FALSE;
                sPtr = 0;
                NumBase = 10;
                Signed = FALSE;
                ParseType = ParseIncomplete;
                while (ParseType == ParseIncomplete)
                    switch (tolower(ch = *(ptr++)))
                    {
                        case 0:
                            ptr--;
                            ParseType = ParseNonsense;
                            break;
                        case '-':
                            LeftJustify = TRUE;
                            break;
                        case '0':
                        case '1':
                        case '2':
                        case '3':
                        case '4':
                        case '5':
                        case '6':
                        case '7':
                        case '8':
                        case '9':
                            MinWidth = MinWidth*10 + ch - '0';
                            if (MinWidth == 0)
                                ZeroFill = TRUE;
                            break;
                        case 'l':
                            Long = TRUE;
                            break;
                        case 'c':
                            sPtr = (PSTR)(param++);
                            sPtr[1] = 0;
                            ParseType = ParseString;
                            break;
                        case 's':
                            sPtr = (PSTR)param[0];
                            param ++;
                            ParseType = ParseString;
                            break;
                        case 'x':
                            NumBase = 16;
                            ParseType = ParseNum;
                            break;
                        case 'd':
                        case 'i':
                            Signed = TRUE;
                        case 'u':
                            ParseType = ParseNum;
                            break;
                        case 'a':
                            ParseType = ParseAddress;
                            break;
                        case '%':
                            ParseType = ParsePercent;
                            break;
                        default:
                            ParseType = ParseNonsense;
                            break;
                    }
                switch (ParseType)
                {
                    case ParsePercent:
                        sPtr = (PSTR)Percent;
                        break;
                    case ParseNonsense:
                        sPtr = (PSTR)PercentQuery;
                        break;

                    case ParseAddress:
                        MinWidth -= 9;
                        FillSpaces(MinWidth,!LeftJustify,ZeroFill);
                        PrintAddress(param,Long,(WORD)(ch=='a'));
                        sPtr = (PSTR)NullMsg;
                        param += 2;
                        break;

                    case ParseNum:
                    {
                        DWORD value;
/*
**  The following modification made to support the Microsoft 32 bit
**  compiler (which is required because the MaCraigor API uses a
**  DLL, which is only supported by the 32 bit compiler).
**
**  The 16 bit compiler aligns stack parameters as 16 bit words.  Thus
**  a byte value passed as a parameter to the printf will be padded by
**  a zero  to take up one word on the stack.  The else statement below
**  would correctly get the byte value and then bump the parameter pointer
**  past the pad char to point at the next parameter.
**
**  But in the 32 bit compiler, parameters are passed as DWORDs, or unsigned
**  longs, and take up 32 bits on the stack.  Thus, the if(Long) is
**  always true for the purposes of incrementing the param pointer.  param
**  is of type LPWORD, so one increment moves it by two bytes.
*/
//                        if (Long)
//                        {
                            value = *((PDWORD)param);
                            param ++;
//                        }
//                        else
//                            value = *(((LPWORD)param)++);
                        if (Signed)
                        {
                            Signed = (value >= (0x8000UL << (16*Long)));
                            if (Signed)
                            {
                                value = 0-value;
                                if (!Long)
                                    value &= 0x7FFF;
                            }
                        }

                        sPtr = &NumString[sizeof(NumString)];
                        *(--sPtr) = 0;
                        do {
                            *(--sPtr) = HexDigit(value % NumBase,ch);
                            value /= NumBase;
                        } while (value > 0);
                        if (Signed)
                            *(--sPtr) = '-';
                    }
                }
                MinWidth -= strlen(sPtr);
                FillSpaces(MinWidth,!LeftJustify,ZeroFill);
                while (*sPtr != 0)
                    writechar(*(sPtr++));
                FillSpaces(MinWidth,LeftJustify,FALSE);
                break;

            default:
                writechar(ch);
				// if this was a line feed, also add a Carriage return
				if(ch == 0xa) writechar(0xd);
                break;
        }
    }
}

/////////////////////////////////////////////////////////////////////////
//   iprintf - Internal printf for recursion, doesn't alter writechar
//
static void iprintf(char * format, ...)
{
    vpprintf(writechar,(PCSTR *)&format);
}

/////////////////////////////////////////////////////////////////////////
//   printf - 
//
void dprintf(char * format, ...)
{
    vpprintf(output_char,(PCSTR *)&format);
}

/* character write routine for sprintf */
void putspbuf(char data)
{
	sprintbuf[sbindex++] = data;
}
/////////////////////////////////////////////////////////////////////////
//   sprintf - 
//
void sprintf(char * buffer, char * format, ...)
{
	sbindex = 0;
    vpprintf((WRITECHAR)putspbuf,(PCSTR *)&format);
	putspbuf(0);
	strcpy(buffer,sprintbuf);
}
