#ifndef CYGONCE_HAL_PLATFORM_SETUP_H
#define CYGONCE_HAL_PLATFORM_SETUP_H

/*=============================================================================
//
//		hal_platform_setup.h
//
//		Platform specific support for HAL (assembly code)
//
//#####ECOSGPLCOPYRIGHTBEGIN####
//## -------------------------------------------
//## This file is part of eCos, the Embedded Configurable Operating System.
//## Copyright (C) 1998, 1999, 2000, 2001, 2002 Red Hat, Inc.
//##
//## eCos is free software; you can redistribute it and/or modify it under
//## the terms of the GNU General Public License as published by the Free
//## Software Foundation; either version 2 or (at your option) any later version.
//##
//## eCos is distributed in the hope that it will be useful, but WITHOUT ANY
//## WARRANTY; without even the implied warranty of MERCHANTABILITY or
//## FITNESS FOR A PARTICULAR PURPOSE.	See the GNU General Public License
//## for more details.
//##
//## You should have received a copy of the GNU General Public License along
//## with eCos; if not, write to the Free Software Foundation, Inc.,
//## 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
//##
//## As a special exception, if other files instantiate templates or use macros
//## or inline functions from this file, or you compile this file and link it
//## with other works to produce a work based on this file, this file does not
//## by itself cause the resulting work to be covered by the GNU General Public
//## License. However the source code for this file must still be made available
//## in accordance with section (3) of the GNU General Public License.
//##
//## This exception does not invalidate any other reasons why a work based on
//## this file might be covered by the GNU General Public License.
//##
//## Alternative licenses for eCos may be arranged by contacting Red Hat, Inc.
//## at http://sources.redhat.com/ecos/ecos-license/
//## -------------------------------------------
//#####ECOSGPLCOPYRIGHTEND####
//#####DESCRIPTIONBEGIN####
//
// Author(s):	usteinkohl
// Contributors:
// Date:		14th January 2003
// Purpose:		KaRo TRITON platform specific support routines
// Description:
// Usage:		#include <cyg/hal/hal_platform_setup.h>
//
//####DESCRIPTIONEND####
//
//===========================================================================*/

#include <pkgconf/system.h>		// System-wide configuration info
#include CYGBLD_HAL_PLATFORM_H		// Platform specific configuration
#include <cyg/hal/hal_triton.h>		// Platform specific hardware definitions
#include <cyg/hal/hal_mmu.h>		// MMU definitions

// Define macro used to diddle the LEDs during early initialization.
// Can use r0+r1.  Argument in \x.
#define CYGHWR_LED_MACRO nop ;

// The main useful output of this file is PLATFORM_SETUP1: it invokes lots
// of other stuff (may depend on RAM or ROM start).	The other stuff is
// divided into further macros to make it easier to manage what's enabled
// when.

#if defined(CYG_HAL_STARTUP_ROM)
#define PLATFORM_SETUP1 _platform_setup1
//#define CYGHWR_HAL_ARM_HAS_MMU
#else
#define PLATFORM_SETUP1
#endif

#define RAM_BASE	0xa0000000
#define	DRAM_SIZE	(64*1024*1024)		// max size of available SDRAM
#define	DCACHE_SIZE	(32*1024)		// size of the Dcache
#define DCACHE_FLUSH_AREA (RAM_BASE+DRAM_SIZE)	// NB: needs page table support

#if 0
#define CPSR_IRQ_DISABLE	0x80	// IRQ disabled when =1
#define CPSR_FIQ_DISABLE	0x40	// FIQ disabled when =1
#define CPSR_FIQ_MODE		0x11
#define CPSR_IRQ_MODE		0x12
#define CPSR_SUPERVISOR_MODE	0x13
#define CPSR_UNDEF_MODE		0x1B

#define CPSR_MODE_BITS		0x1F

#endif

#define MMU_Control_BTB 0x800

// Reserved area for battery backup SDRAM memory test
// This area is not zeroed out by initialization code
#define	SDRAM_BATTERY_TEST_BASE	0xA1FFFFF0	// base address of last 16 memory locations in a 32MB SDRAM

	.macro MEM_TEST
		nop
	.endm

	// Trigger the logic analyzer by writing a particular
	// address, and triggering on that address.
	.macro TRIGGER_LA_ON_ADDRESS address, reg0, reg1
	mrc	p15, 0, \reg0, c1, c0, 0	// read ARM control register
	//	CPWAIT	\reg0
	ldr	\reg1, =\address
	str	\reg0, [\reg1]
	.endm

	// Delay a bit
	.macro DELAY_FOR cycles, reg0
	ldr	\reg0, =\cycles
	subs	\reg0, \reg0, #1
	subne	pc, pc, #0xc
	.endm

	// wait for coprocessor write complete
	.macro CPWAIT reg
	mrc	p15, 0, \reg, c2, c0, 0
	mov	\reg, \reg
	sub	pc, pc, #4
	.endm

	// blink some times on GPIO10
	.macro BLINK10 count
	ldr	r4, =\count
220961:
	ldr	r1, =0x00000400		// we use GPIO10 for controlling the debug LED

	ldr	r0, =GPCRx
	str	r1, [r0]		// switch the LED on

	ldr	r2, =0x00500000		// wait some time
	mov	r3, #1
998:
	sub	r2, r2, r3
	cmp	r2, #0
	bne	998b

	ldr	r0, =GPSRx		// switch the LED off
	str	r1, [r0]

	ldr	r2, =0x00500000		// wait some time
	mov	r3, #1
998:
	sub	r2, r2, r3
	cmp	r2, #0
	bne	998b

	sub	r4, r4, r3
	cmp	r4, #0
	bne	220961b
	.endm

	.macro TRITON_SET_REFR_VAL
	// TRITON specific, DRAM specific !!!!!!!!!!!!!
	// example:
	// Refresh period for 4096 rows = 64 ms (47 memory cycles) for 99,5 MHz MCLK
	// DRI = 64 ms * 99,5 MHz / (4096 * 32) = 48

	ldr r3, =MDREFR
	ldr r2, [r3]			// read MDREFR value
	ldr	r4, =0xFFF
	bic r2, r2, r4			// clear out value in DRI
#ifdef TRITON_DRAM32_64
	// DRI = 64 ms * 99,5 MHz / (8192 * 32) = 24   (0x18)
	orr r2, r2, #0x00000018		// put in a valid SDRAM Refresh Interval (DRI)
#endif
#ifdef TRITON_DRAM16_16_MOBILE
	// DRI = 64 ms * 99,5 MHz / (4096 * 32) = 48
	orr r2, r2, #0x00000030		// put in a valid SDRAM Refresh Interval (DRI)
#endif
#ifdef TRITON_DRAM16_32_MOBILE
	// DRI = 64 ms * 99,5 MHz / (8192 * 32) = 24
	orr r2, r2, #0x00000018		// put in a valid SDRAM Refresh Interval (DRI)
#endif

	str	r2, [r3]		// store it

	.endm

	.macro TRITON_CONFIG_SDRAM_BANKS

	// Banks configured for 64-Mbit SDRAM devices (12(13) row x 9 col x 4 internal banks)
	// CAS latency = 3, 32/16-bit bus width,

	ldr r3, =MDCNFG			// sdram config -- sdram should remain disabled !!!!!

#ifdef TRITON_DRAM32_64
	ldr r2, =0x00000ac8		// 13 rows
#endif
#ifdef TRITON_DRAM16_16_MOBILE
	ldr r2, =0x00000aac		// 12 rows / 9 col / 4 banks / 16 bit / CL3
#endif
#ifdef TRITON_DRAM16_32_MOBILE
	ldr r2, =0x00000acc		// 13 rows / 9 col / 4 banks / 16 bit / CL3
#endif

	str r2, [r3]
	.endm

	// form a first-level section entry
	.macro FL_SECTION_ENTRY base,x,ap,p,d,c,b
	.word (\base << 20) | (\x << 12) | (\ap << 10) | (\p << 9) |\
	(\d << 5) | (\c << 3) | (\b << 2) | 2
	.endm

	// form a first-level page table entry
	.macro FL_PT_ENTRY base,d
	// I wanted to use logical operations here, but since I am using symbols later
	// to fill in the parameters, I had to use addition to force the assembler to
	// do it right
	.word \base + (\d << 5) + 1
	.endm

	// form a second level small page entry
	.macro SL_SMPAGE_ENTRY base,ap3,ap2,ap1,ap0,c,b
	.word (\base << 12) | (\ap3 << 10) | (\ap2 << 8) | (\ap1 << 6) |\
	(\ap0 << 4) | (\c << 3) | (\b << 2) | 2
	.endm

	// form a second level extended small page entry
	.macro SL_XSMPAGE_ENTRY base,x,ap,c,b
	.word (\base << 12) | (\x << 6) | (\ap << 4) | (\c << 3) | (\b << 2) | 3
	.endm

	// start of platform setup
	.macro _platform_setup1

	// This is where we wind up immediately after reset. On the CYCLONE, we have
	// to jump around a hole in flash which runs from 0x00001000 - 0x0001fff.
	// We might not have to do this for TRITON
	// The start of _platform_setup1 will be below 0x1000 and since we need to
	// align the mmu table on a 16k boundary, we just branch around the page
	// table which we will locate at FLASH_BASE+0x4000.
	b _real_platform_setup

	.p2align 13
	// the following alignment creates the mmu table at address 0x4000.
	mmu_table:

	// 1MB of FLASH with i80312 MMRs mapped in using 4K small pages so we can
	// set the access permission on flash and memory-mapped registers properly.
	FL_PT_ENTRY mmu_table_flashbase, 0

	// Remaining 63MB of FLASH area (Static Chip select 0)
	//	rw, cacheable, non-bufferable
	.set	__base, 1
	.rept	0x040-0x001
	FL_SECTION_ENTRY __base, 0, 3, 0, 0, 1, 0
	.set	__base, __base+1
	.endr

	// nothing interesting here, static chip select area 1 (Address Translation)
	.rept	0x080 - 0x040
	FL_SECTION_ENTRY __base, 0, 3, 0, 0, 0, 0
	.set	__base, __base+1
	.endr

	// nothing interesting here, static chip select area 2 (Address Translation)
	.rept	0x0C0 - 0x080
	FL_SECTION_ENTRY __base, 0, 3, 0, 0, 0, 0
	.set	__base, __base+1
	.endr

	// nothing interesting here, static chip select area 3 (Address Translation)
	.rept	0x100 - 0x0C0
	FL_SECTION_ENTRY __base, 0, 3, 0, 0, 0, 0
	.set	__base, __base+1
	.endr

	// nothing interesting here, static chip select area 4 (Address Translation)
	.rept	0x140 - 0x100
	FL_SECTION_ENTRY __base, 0, 3, 0, 0, 0, 0
	.set	__base, __base+1
	.endr

	// nothing interesting here, static chip select area 5 (Address Translation)
	.rept	0x180 - 0x140
	FL_SECTION_ENTRY __base, 0, 3, 0, 0, 0, 0
	.set	__base, __base+1
	.endr

	// nothing interesting here (Address Translation)
	.rept	0xA00 - 0x180
	FL_SECTION_ENTRY __base, 0, 3, 0, 0, 0, 0
	.set	__base, __base+1
	.endr

	// up to 64MB SDRAM
	//	x=c=b=1
	// first 1MB mapped by second level table
	FL_PT_ENTRY mmu_table_rambase, 0
	.set	__base, __base+1

	// remainder of SDRAM mapped 1-to-1
	.rept	0xC00 - 0xA01
	FL_SECTION_ENTRY __base, 1, 3, 1, 0, 1, 1
	.set	__base, __base+1
	.endr

	// Cache flush region.
	// Don't need physical memory, just a cached area.
	.rept	0xD00 - 0xC00
	FL_SECTION_ENTRY __base, 0, 3, 0, 0, 1, 1
	.set	__base, __base+1
	.endr

	// Invalid
	.rept	0x1000 - 0xD00
	.word 0
	.set	__base, __base+1
	.endr

	// Immediately after the above table (at 0x8000) is the
	// second level page table which breaks up the lowest 1MB
	// of physical memory into 4KB sized virtual pages.
	mmu_table_flashbase:
	// Virtual address 0 (Flash boot code).
	// Map 4k page at 0x00000000 virt --> 0xA0000000 physical
	// This allows us to have a writable vector table.
	//	Read-Write, cacheable, bufferable
	SL_XSMPAGE_ENTRY 0xa0000, 1, 3, 1, 1

	// Virtual address 0x1000 (Memory mapped registers)
	// Map 1-to-1, but don't cache or buffer
	//	Read-Write, non-cacheable, non-bufferable
	.set	__base, 1
	SL_SMPAGE_ENTRY __base, 3, 3, 3, 3, 0, 0
	.set	__base, __base+1

	// Virtual address 0x2000-0x100000 (remainder of flash1)
	//	Read-Write, cacheable, non-bufferable
	.rept	0x100 - 0x2
	SL_SMPAGE_ENTRY __base, 3, 3, 3, 3, 1, 0
	.set	__base, __base+1
	.endr

	// Now is the second level table for the first megabyte
	// of DRAM.
	mmu_table_rambase:
	// Map 4k page at 0xa0000000 virt --> 0x00000000 physical
	//	Read-Write, cacheable, non-bufferable
	SL_SMPAGE_ENTRY 0x00000, 3, 3, 3, 3, 1, 0
	.set	__base, __base+1

	// Map remainder of first meg of SDRAM
	//	Read-Write, cacheable, non-bufferable
	.set	__base, 0xA0001
	.rept	0x100 - 0x1
	SL_XSMPAGE_ENTRY __base, 1, 3, 1, 1
	.set	__base, __base+1
	.endr

_real_platform_setup://Angel SDRAM init code follows

	// if we come out of sleep mode, check if there is a pointer
	// from the os, and jump to it if so

	ldr r0, =RCSR		// check reset source
	ldr	r1, [r0]
	and r1, r1, #4
	cmp r1, #4
	bne 1923f

	ldr r2, =PSPR		// check if there is a pointer from the operating system
	ldr r1, [r2]
	cmp r1, #0
	beq 1923f

	ldr r3, =0x0000000f		// reset RCSR
	str r3, [r0]

	// get SDRAM out of Self Refresh
	TRITON_SET_REFR_VAL
	TRITON_CONFIG_SDRAM_BANKS

	ldr r3, =MDREFR
	ldr r2, [r3]		// read MDREFR value
	bic r2, r2, #0x03800000	// clear all Free Running Clocks
	orr r2, r2, #0x00010000	// assert MDREFR:K1RUN
	bic r2, r2, #0x00020000	// clear MDREFR:K1DB2 -> SDRAM-CLK = MEMCLK
	str r2, [r3]		// change from "self-refresh and clock-stop" to "self-refresh"
	ldr r2, [r3]		// read MDREFR value (to make sure the previous value stuck) (this is in Tricia's code)

	bic r2, r2, #0x00400000	// clear self-refresh bit
	str r2, [r3]		// change from "self-refresh" to "Power Down"
	orr r2, r2, #0x00008000	// assert MDREFR:E1PIN
	str r2, [r3]		// change from "Power Down" to "PWRDWNX"
	nop
	nop				// no write required to change from "PWRDWNX" to "NOP"

	ldr r3, =MDCNFG		// sdram config -- sdram enable
	ldr r2, [r3]
	orr r2, r2, #0x00000001	// enable appropriate banks
	str r2, [r3]

	// initialize CPSR (machine state register)
	mov	r0, #(CPSR_IRQ_DISABLE|CPSR_FIQ_DISABLE|CPSR_SUPERVISOR_MODE)
	msr	cpsr, r0

	/*	Set up the stack pointer to a fixed value */
	ldr	r3, =0xa0700000
	mov	sp, r3

	ldr r2, =PSPR		// get pointer
	ldr r1, [r2]
	mov r15, r1			// jump to pointer

1923:
	//	must set the GPIOs up before any chip selects will work
	//GPCRx = 0xffffffff put a 0 on any of the GPIOs (0=unchanged, 1=drive 0)
	// reset all first
	ldr	r0, =GPCRx
	ldr	r1, =0xffffffff
	str	r1, [r0]
	//GPCRy = 0xffffffff
	ldr	r0, =GPCRy
	ldr	r1, =0xffffffff
	str	r1, [r0]
	//GPCRz = 0xffffffff
	ldr	r0, =GPCRz
	ldr	r1, =0xffffffff
	str	r1, [r0]

	// GPSRx = 0x01800000 put a 1 on any of the GPIOs (0=unchanged, 1=drive 1)
	ldr	r0, =GPSRx
	ldr	r1, =0x01800000
#ifdef TRITON_CS2_SMSC
	ldr	r1, =0x01808000
#endif

#ifdef TRITON_CS2_8900
	ldr	r1, =0x01808000
#endif
	str	r1, [r0]

	//GPSRy = 0x70ff0000
	ldr	r0, =GPSRy
	ldr	r1, =0x70ff0800
#ifdef TRITON_USE_STUART
	ldr	r2, =0x00008000
	orr	r1, r1, r2
#endif
	str	r1, [r0]

	//GPSRz = 0x00000000
	ldr	r0, =GPSRz
	ldr	r1, =0x00000000
	str	r1, [r0]

	//GPDRx = 0x01800800 put the GPIOs in the correct direction (0=in, 1=out)
	ldr	r0, =GPDRx
	ldr	r1, =0x01800000		//vorher 0x01800400
#ifdef TRITON_CS2_SMSC
	ldr	r1, =0x01808800
#endif
#ifdef TRITON_CS2_8900
	ldr	r1, =0x01808000
#endif
#if (TRITON_STK==1)
	orr	r1, r1, #0x00000800
#endif
#ifdef GPIO_10_OUT
	orr	r1, r1, #0x00000400
#endif
	str	r1, [r0]

	//GPDRy = 0x7cff0280
	ldr	r0, =GPDRy
	ldr	r1, =0x7cff0a80
#ifdef TRITON_USE_STUART
	ldr	r2, =0x00008000
	orr	r1, r1, r2
#endif
	str	r1, [r0]
	//GPDRz = 0x00000000
	ldr	r0, =GPDRz
	ldr	r1, =0x00000000
	str	r1, [r0]

	//GAFR0x = 0x00000000  setup the alternate functions (00=normal, 01=alt fuct 1, etc)
	ldr	r0, =GAFR0x
	ldr	r1, =0x00000000
#ifdef TRITON_CS2_SMSC
	ldr	r1, =0x80000000
#endif
#ifdef TRITON_CS2_8900
	ldr	r1, =0x80000000
#endif
	str	r1, [r0]

	//GAFR1x = 0x00128140
	ldr	r0, =GAFR1x
	ldr	r1, =0x0012814
	str	r1, [r0]

	//GAFR0y = 0x00988050
	ldr	r0, =GAFR0y
	ldr	r1, =0x00988050
#ifdef TRITON_USE_STUART
	ldr	r2, =0x60000000
	orr	r1, r1, r2
#endif
	str	r1, [r0]

	//GAFR1y = 0x0005aaaa
	ldr	r0, =GAFR1y
	ldr	r1, =0x0005aaaa
	str	r1, [r0]

	//GAFR0z = 0x00000000
	ldr	r0, =GAFR0z
	ldr	r1, =0x00000000
	str	r1, [r0]

	//GAFR1z = 0x00000000
	ldr	r0, =GAFR1z
	ldr	r1, =0x00000000
	str	r1, [r0]

	//PSSR = 0x20 clear the RDH and PH bit in the PSSR
	ldr	r0, =PSSR
	ldr	r1, =0x30
	str	r1, [r0]

	ldr r3, =PCFR		// clear PCFR[FS] and PCFR[FP]
	ldr r2, [r3]
	bic r2, r2, #0x00000006
	str r2, [r3]
#ifdef GPIO_10_OUT
	BLINK10 3
#endif
	MRC	p15, 0, r0, c0, c0, 0		// read the ID reg ....	0x69052100
	// and display it if possible
	// not possible for TRITON

	// turn everything off
	mov	r0, #0x78
	mcr	p15, 0, r0, c1, c0, 0	// caches off -- MMU off or ID map

	mcr p15, 0, R0, c7, c7, 0	// Invalidate the I & D cache, mini- d cache, and BTB

	//MCR	p15, 0, r0, c7, c5, 0	// Invalidate the instruction cache and branch target buffer
	//MCR	p15, 0, r0, c7, c6, 0	// Invalidate the data cache and mini-data cache

	MCR p15, 0, r0, c7, c10, 4	// Drain write buffer -- r0 ignored

					// CPWAIT macro
	MRC p15, 0, R0, c2, c0, 0	// arbitrary read of CP15
	MOV R0, R0			// wait for it
	SUB PC, PC, #4			// branch to next instruction

	nop
	nop
	nop
	nop
// setup manager access, interrupts, etc.
// grant manager access to all domains
	mvn	r0, #0		// all 1s
	mcr	p15, 0, r0, c3, c0, 0
					// all IRQs should be masked to prevent spurious IRQs
	ldr	r3, =ICMR		// pending interrupts are masked from becoming active
	mov	r2, #0
	str	r2, [r3]

	ldr	r3, =ICLR		// route all interrupts to CPU IRQ ( not to FIQ )
	mov	r2, #0
	str	r2, [r3]

	ldr	r3, =ICCR		// only enabled and unmasked interrupts bring core out of idle
	mov	r2, #1
	str	r2, [r3]

// Turn on the RTC circuit (if you want)

	ldr	r1, =OSCC		// oscillator config reg
	// turn on the 32.768 KHz clock for RTC, etc.
	mov	r0, #0x2
	str	r0, [r1]

	// Wait 200 usec
	ldr	r3, =OSCR	// reset the OS Timer Count to zero
	mov	r2, #0
	str	r2, [r3]
	ldr	r4, =0x300	// really 0x2E1 is about 200usec, so 0x300 should be plenty

10:
	ldr	r2, [r3]
	cmp	r4, r2
	bgt	10b

// CS0	: RDF=14(15), RDN=4 , RRR=2, 32 bits Flash
// Suitable for 128-Mbit StrataFlash (Tcyc = 150 ns) and 100MHz MEMCLK
	ldr	r3, =MSC0		// low  - bank 0 Flash
#ifdef TRITON_FLASH32_32
	ldr	r2, =0x7ff024d0
#endif
#ifdef TRITON_FLASH16_16
	ldr	r2, =0x7ff024d8
#endif

#ifdef TRITON_CS2_SMSC
	ldr	r4, =0x0000ffff
	and	r2, r2, r4
	ldr	r4, =0x12440000
	orr	r2, r2, r4
#endif
#ifdef TRITON_CS2_8900
	ldr	r4, =0x0000ffff
	and	r2, r2, r4
	ldr	r4, =0x12dc0000
	orr	r2, r2, r4
#endif
	str	r2, [r3]
	ldr	r2, [r3]		// need to read it back to make sure the value latches (see MSC section of manual)

	ldr	r3, =MSC1
	ldr	r2, =0x7ff07ff0
	str	r2, [r3]
	ldr	r2, [r3]		// need to read it back to make sure the value latches (see MSC section of manual)

	ldr	r3, =MSC2
	ldr	r2, =0x7ff07ff0
	str	r2, [r3]
	ldr	r2, [r3]		// need to read it back to make sure the value latches (see MSC section of manual)

	// ********************************************************************
	// Disable (mask) all interrupts at the interrupt controller

	// clear the interrupt level register (use IRQ, not FIQ)
	mov	r1, #0
	ldr	r2,  =ICLR
	str	r1,  [r2]

	// mask all interrupts at the controller
	ldr	r2,  =ICMR
	str	r1,  [r2]

	// make sure the DCACHE is off
	mov	r0, #0x78		// turn everything off
	mcr	p15, 0, r0, c1, c0, 0	// caches off, MMU off, etc.

	TRITON_SET_REFR_VAL

	ldr	r3, =SXCNFG
	mov	r2, #0x0		// since Triton does not have SMROM, make register = 0
	str	r2, [r3]		// store it

	ldr	r3, =MDREFR
	ldr	r2, [r3]		// read MDREFR value
	bic	r2, r2, #0x03800000	// clear all Free Running Clocks
	orr	r2, r2, #0x00010000	// assert MDREFR:K1RUN
	bic	r2, r2, #0x00020000	// clear MDREFR:K1DB2 -> SDRAM-CLK = MEMCLK

	str	r2, [r3]		// change from "self-refresh and clock-stop" to "self-refresh"
	ldr	r2, [r3]		// read MDREFR value (to make sure the previous value stuck) (this is in Tricia's code)

	bic	r2, r2, #0x00400000	// clear self-refresh bit
	str	r2, [r3]		// change from "self-refresh" to "Power Down"
	orr	r2, r2, #0x00008000	// assert MDREFR:E1PIN
	str	r2, [r3]		// change from "Power Down" to "PWRDWNX"
	nop
	nop			// no write required to change from "PWRDWNX" to "NOP"

	TRITON_CONFIG_SDRAM_BANKS

	//---- Wait 200 usec
	ldr	r3, =OSCR	// reset the OS Timer Count to zero
	mov	r2, #0
	str	r2, [r3]
	ldr	r4, =0x300	// really 0x2E1 is about 200usec, so 0x300 should be plenty
10:
	ldr	r2, [r3]
	cmp	r4, r2
	bgt	10b

	ldr	r3, =SDRAM_B0
	mov	r2, #9		// now must do 8 refresh or CBR commands before the first access
10:				// no we must do 9 because of errata 116
	str	r3, [r3]
	subs	r2, r2, #1
	bne	10b

	ldr	r3, =MDCNFG		// sdram config -- sdram enable
	ldr	r2, [r3]
	orr	r2, r2, #0x00000001	// enable appropriate banks
	str	r2, [r3]

	ldr	r3, =MDMRS		// write the MDMRS
	mov	r2, #0			// the writable bits will be written as a 0 (Tricia's code writes 0x00320032)
	str	r2, [r3]
#ifdef TRITON_DRAM16_16_MOBILE
	ldr	r3, =MDMRSLP
	ldr	r2, =0x0000a000
	str	r2, [r3]
#endif
#ifdef TRITON_DRAM16_32_MOBILE
	ldr	r3, =MDMRSLP
	ldr	r2, =0x0000c000
	str	r2, [r3]
#endif
	//---- Wait 600 usec
	ldr	r3, =OSCR		// reset the OS Timer Count to zero
	mov	r2, #0
	str	r2, [r3]
	ldr	r4, =0x900		// really 0x2E1 is about 200usec, so 0x300 should be plenty
10:
	ldr	r2, [r3]
	cmp	r4, r2
	bgt	10b

	MEM_TEST
#if 0
	/*	Set up the stack pointer to a fixed value */
	ldr	r3, =0xa0700000
	mov	sp, r3

	// if we come out of sleep mode, check if there is a pointer
	// from the os, and jump to it if so

	ldr	r0, =RCSR		// check reset source
	ldr	r1, [r0]
	and	r1, r1, #4
	cmp	r1, #4
	bne	1923f

	ldr	r2, =PSPR		// check if there is a pointer from the operating system
	ldr	r1, [r2]
	cmp	r1, #0
	beq	1923f

	ldr	r3, =0x0000000f	// reset RCSR
	str	r3, [r0]

	// initialize CPSR (machine state register)
	mov	r0, #(CPSR_IRQ_DISABLE|CPSR_FIQ_DISABLE|CPSR_SUPERVISOR_MODE)
	msr	cpsr, r0

	mov	r15, r1		// jump to pointer
1923:
#endif

_sdram_init_end:
//
//
// Change Frequency!!!
//
//
#if 0
	ldr	r4, =0x145	// PxBus = 165,9 MHz MEM/LCD Clock = 165,9 MHz SDRAM-Clock 83 MHz
	ldr	r2, =CCCR	// Core Clock Config Reg
	str	r4, [r2]	// set speed

	// set to Frequency Change Mode
	mov	r0, #0x3
	mcr	p14, 0, r0, c6, c0, 0	// write to the CCLKCFG coprocessor reg
					// no TURBO-mode is set here
#endif
	// Enable access to all coprocessor registers
	ldr	r0, =0x2001		// enable access to all coprocessors
	mcr	p15, 0, r0, c15, c1, 0

	mcr	p15, 0, r0, c7, c10, 4	// drain the write & fill buffers
	CPWAIT	r0

	mcr	p15, 0, r0, c7, c7, 0	// flush Icache, Dcache and BTB
	CPWAIT	r0

	mcr	p15, 0, r0, c8, c7, 0	// flush instuction and data TLBs
	CPWAIT	r0

	// Enable the Icache
	mrc	p15, 0, r0, c1, c0, 0
	orr	r0, r0, #MMU_Control_I
	mcr	p15, 0, r0, c1, c0, 0
	CPWAIT	r0

	// Set the TTB register
	ldr	r0, =mmu_table
	mcr	p15, 0, r0, c2, c0, 0

	// Enable permission checks in all domains
	ldr	r0, =0x55555555
	mcr	p15, 0, r0, c3, c0, 0

	// Enable the MMU
	mrc	p15, 0, r0, c1, c0, 0
	orr	r0, r0, #MMU_Control_M
	orr	r0, r0, #MMU_Control_R
	mcr	p15, 0, r0, c1, c0, 0
	CPWAIT	r0

	mcr	p15, 0, r0, c7, c10, 4	// drain the write & fill buffers
	CPWAIT	r0

	// Enable the Dcache
	mrc	p15, 0, r0, c1, c0, 0
	orr	r0, r0, #MMU_Control_C
	mcr	p15, 0, r0, c1, c0, 0
	CPWAIT	r0

	// Enable the BTB
	mrc	p15, 0, r0, c1, c0, 0
	orr	r0, r0, #MMU_Control_BTB
	mcr	p15, 0, r0, c1, c0, 0
	CPWAIT	r0

	// clean/drain/flush the main Dcache
	mov	r1, #DCACHE_FLUSH_AREA	// use a CACHEABLE area of
					// the memory map above SDRAM
	mov	r0, #1024		// number of lines in the Dcache
20:
	mcr	p15, 0, r1, c7, c2, 5	// allocate a Dcache line
/* increment the address to the next cache line */
	add	r1, r1, #32
// decrement the loop count
	subs	r0, r0, #1		// decrement the loop count
	bne	20b

	// clean/drain/flush the mini Dcache
	ldr	r2, =(DCACHE_FLUSH_AREA+DCACHE_SIZE) // use a CACHEABLE area of
					// the memory map above SDRAM
	mov	r0, #64			// number of lines in the mini Dcache
21:
	mcr	p15, 0, r2, c7, c2, 5	// allocate a Dcache line
	add	r2, r2, #32		// increment the address to
					// the next cache line
	subs	r0, r0, #1		// decrement the loop count
	bne	21b

	mcr	p15, 0, r0, c7, c6, 0	// flush Dcache
	CPWAIT	r0

	mcr	p15, 0, r0, c7, c10, 4	// drain the write & fill buffers
	CPWAIT	r0
#if 1
	mov	r0, #0x1000000
1:
	subs	r0, r0, #1
	bne	1b
#endif
	// Save SDRAM size
	ldr	r1, =hal_dram_size	// [see hal_intr.h]
	str	r8, [r1]

	// Move mmu tables into RAM so page table walks by the cpu
	// don't interfere with FLASH programming.
	ldr	r0, =mmu_table
	mov	r4, r0
	add	r2, r0, #0x4800		// End of tables
	mov	r1, #RAM_BASE
	orr	r1, r1, #0x4000		// RAM tables
	mov	r5, r1

	// first, fixup physical address to second level
	// table used to map first 1MB of flash.
	ldr	r3, [r0], #4
	sub	r3, r3, r4
	add	r3, r3, r5
	str	r3, [r1], #4
	// everything else can go as-is
1:
	ldr	r3, [r0], #4
	str	r3, [r1], #4
	cmp	r0, r2
	bne	1b

	// go back and fixup physical address to second level
	// table used to map first 1MB of SDRAM.
	add	r1, r5, #(0xA00 * 4)
	ldr	r0, [r1]		// entry for first 1MB of DRAM
	sub	r0, r0, r4
	add	r0, r0, r5
	str	r0, [r1]		// store it back

	// Flush the cache
	mov	r0, #DCACHE_FLUSH_AREA	/* cache flush region */
	add	r1, r0, #0x8000	/* 32KB cache	*/
667:
	mcr	p15, 0, r0, c7, c2, 5	/* allocate a line	*/
	add	r0, r0, #32		/* 32 bytes/line	*/
	teq	r1, r0
	bne	667b
	mcr	p15, 0, r0, c7, c6, 0	/* invalidate data cache */
	/* cpuwait */
	mrc	p15, 0, r1, c2, c0, 0	/* arbitrary read	*/
	mov	r1, r1
	sub	pc, pc, #4
	mcr	p15, 0, r0, c7, c10, 4
	/* cpuwait */
	mrc	p15, 0, r1, c2, c0, 0	/* arbitrary read	*/
	mov	r1, r1
	sub	pc, pc, #4
	nop

	// Set the TTB register to DRAM mmu_table
	mov	r0, r5
	mov	r1, #0
	mcr	p15, 0, r1, c7, c5, 0	// flush I cache
	mcr	p15, 0, r1, c7, c10, 4	// drain WB
	mcr	p15, 0, r0, c2, c0, 0	// load page table pointer
	mcr	p15, 0, r1, c8, c7, 0	// flush TLBs
	CPWAIT	r0

	//Disable software and data breakpoints
	mov	r0, #0
	mcr	p15, 0, r0, c14, c8, 0  // ibcr0
	mcr	p15, 0, r0, c14, c9, 0  // ibcr1
	mcr	p15, 0, r0, c14, c4, 0  // dbcon

	//Enable all debug functionality
	mov	r0, #0x80000000
	mcr	p14, 0, r0, c10, c0, 0  // dcsr
#if 0
220961:
	ldr	r1, =0x00000800		// we use GPIO23 for controlling the debug LED

	ldr	r0, =GPCRx
	str	r1, [r0]		// switch the LED on

	ldr	r2, =0x00500000		// wait some time
	mov	r3, #1
998:
	sub	r2, r2, r3
	cmp	r2, #0
	bne	998b

	ldr	r0, =GPSRx		// switch the LED off
	str	r1, [r0]

	ldr	r2, =0x00500000		// wait some time
	mov	r3, #1
998:
	sub	r2, r2, r3
	cmp	r2, #0
	bne	998b

	b	220961b
#endif
	mov	r0, #0

	//---- Wait 200 usec
	ldr	r3, =OSCR			// reset the OS Timer Count to zero
	mov	r2, #0
	str	r2, [r3]
	ldr	r4, =0x300		// really 0x2E1 is about 200usec, so 0x300 should be plenty
10:
	add	r0, r0, #1
	ldr	r2, [r3]
	cmp	r4, r2
	bgt	10b

	ldr	r2, =0xa0800000
	str	r0, [r2]
	.endm	// _platform_setup1
/*----------------------------------------------------------------------*/
/* end of hal_platform_setup.h						*/
#endif /* CYGONCE_HAL_PLATFORM_SETUP_H */
