//==========================================================================
//
//      redboot_cmds.c
//
//      KaRo TRITON270 [platform] specific RedBoot commands
//
//==========================================================================
//#####ECOSGPLCOPYRIGHTBEGIN####
//## -------------------------------------------
//## This file is part of eCos, the Embedded Configurable Operating System.
//## Copyright (C) 1998, 1999, 2000, 2001, 2002 Red Hat, Inc.
//##
//## eCos is free software; you can redistribute it and/or modify it under
//## the terms of the GNU General Public License as published by the Free
//## Software Foundation; either version 2 or (at your option) any later version.
//##
//## eCos is distributed in the hope that it will be useful, but WITHOUT ANY
//## WARRANTY; without even the implied warranty of MERCHANTABILITY or
//## FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
//## for more details.
//##
//## You should have received a copy of the GNU General Public License along
//## with eCos; if not, write to the Free Software Foundation, Inc.,
//## 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
//##
//## As a special exception, if other files instantiate templates or use macros
//## or inline functions from this file, or you compile this file and link it
//## with other works to produce a work based on this file, this file does not
//## by itself cause the resulting work to be covered by the GNU General Public
//## License. However the source code for this file must still be made available
//## in accordance with section (3) of the GNU General Public License.
//##
//## This exception does not invalidate any other reasons why a work based on
//## this file might be covered by the GNU General Public License.
//##
//## Alternative licenses for eCos may be arranged by contacting Red Hat, Inc.
//## at http://sources.redhat.com/ecos/ecos-license/
//## -------------------------------------------
//#####ECOSGPLCOPYRIGHTEND####
//==========================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):    gthomas
// Contributors: gthomas
//               Richard Panton <richard.panton@3glab.com>
// Date:         2001-02-24
// Purpose:      
// Description:  
//              
// This code is part of RedBoot (tm).
//
//####DESCRIPTIONEND####
//
//==========================================================================

#include <redboot.h>

#include <cyg/hal/hal_triton270.h>
#include <cyg/hal/hal_intr.h>
#include <cyg/hal/hal_cache.h>

#include <cyg/hal/hal_io.h>             // IO macros

RedBoot_cmd("mem",
	    "Set a memory location",
	    "[-h|-b] [-a <address>] <data>",
	    do_mem
	    );

void do_mem(int argc, char *argv[])
{
	struct option_info opts[3];
	bool mem_half_word, mem_byte;
	volatile cyg_uint32 *address = NULL;
	cyg_uint32 value;

	init_opts(&opts[0], 'b', false, OPTION_ARG_TYPE_FLG,
		  (void *)&mem_byte, 0, "write a byte");
	init_opts(&opts[1], 'h', false, OPTION_ARG_TYPE_FLG,
		  (void *)&mem_half_word, 0, "write a half-word");
	init_opts(&opts[2], 'a', true, OPTION_ARG_TYPE_NUM,
		  (void *)&address, NULL, "address to write at");
	if (!scan_opts(argc, argv, 1, opts, 3, (void*)&value, OPTION_ARG_TYPE_NUM, "address to set"))
		return;
	if (mem_byte && mem_half_word) {
		diag_printf("*ERR: Should not specify both byte and half-word access\n");
	} else if (mem_byte) {
		*(cyg_uint8*)address = (cyg_uint8)(value & 255);
		diag_printf("  Set 0x%08X to 0x%02X (result 0x%02X)\n", address, value & 255, (int)*(cyg_uint8*)address);
	} else if (mem_half_word) {
		if ((unsigned long)address & 1) {
			diag_printf("*ERR: Badly aligned address 0x%08X for half-word store\n", address);
		} else {
			*(cyg_uint16*)address = (cyg_uint16)(value & 0xffff);
			diag_printf("  Set 0x%08X to 0x%04X (result 0x%04X)\n", address, value & 0xffff, (int)*(cyg_uint16*)address);
		}
	} else {
		if ((unsigned long)address & 3) {
			diag_printf("*ERR: Badly aligned address 0x%08X for word store\n", address);
		} else {
			*address = value;
			diag_printf("  Set 0x%08X to 0x%08X (result 0x%08X)\n", address, value,
				    *address);
		}
	}
}

RedBoot_cmd("clock",
            "Set/Query the system clock speed",
            "[-s <clock> (200/300/400/500/600)]",
            do_clock
	    );

clock_param_t pclktab[] = {
	{91,   45,  91,	 45,  91, 0,  7, 2, 0, 0, 0,  950, 0x02, 1, 1},
	{104, 104, 104, 104,  52, 1,  8, 2, 1, 0, 1,  950, 0x02, 0, 1},
	{156, 104, 104, 104,  52, 1,  8, 6, 1, 1, 1, 1150, 0x06, 0, 1},
	{208, 208, 208, 104, 104, 1, 16, 2, 1, 0, 1, 1150, 0x06, 1, 1},
	{312, 208, 208, 104, 104, 1, 16, 3, 1, 0, 1, 1250, 0x08, 1, 1},
	{416, 208, 208, 104, 104, 1, 16, 4, 1, 0, 1, 1350, 0x0a, 1, 1},
	{520, 208, 208, 104, 104, 1, 16, 5, 1, 0, 1, 1450, 0x0c, 1, 1},
	{624, 208, 208, 104, 104, 1, 16, 6, 1, 0, 1, 1550, 0x0e, 1, 1},
};
clock_param_t *get_clock_rate(void);

int get_clock_table_index(int clock)
{
	int i = 0;
	int deltan;
	int deltap;
	int rindex = 1;
	int maxindex;

	maxindex = sizeof(pclktab) / sizeof(clock_param_t) - 1;
	if (clock < pclktab[0].t_clock) {
		clock = 91;
		rindex = 0;
	} else if (clock > pclktab[maxindex].t_clock) {
		clock = 624;
		rindex = maxindex;
	} else {
		while (i <= maxindex) {
			if (pclktab[i].t_clock < clock) {
				i++;
			} else {
				if (i == maxindex){
					rindex = maxindex;
				} else {
					deltan = pclktab[i].t_clock - clock;
					deltap = pclktab[i + 1].t_clock - clock;
					if (deltan < deltap) {
						rindex = i;
					} else {
						rindex = i + 1;
					}
				}
				break;
			}
		}
	}

	return rindex;
}

void change_core_voltage(unsigned char ind)
{
	unsigned long	value;

	// enable Power I2C
	HAL_READ_UINT32(PCFR, value);
	value |= 0x40;
	HAL_WRITE_UINT32(PCFR, value);

	diag_printf("Core voltage will be set to %d.%d Volts",
		    pclktab[ind].VOLTAGE_VAL / 1000, pclktab[ind].VOLTAGE_VAL % 1000);
	write_i2c_pwr(0x18,  pclktab[ind].DAC_VAL | 0x20);
	diag_printf(".\n");

	// disable Power I2C
	HAL_READ_UINT32(PCFR, value);
	value &= (~0x40);
	HAL_WRITE_UINT32(PCFR, value);
}

clock_param_t *get_clock_rate(void)
{
	unsigned long CCCR_reg, CLKCFG_reg;
	unsigned char clkcfg_t, clkcfg_ht, clkcfg_b;
	unsigned char cccr_l, cccr_2n, cccr_a;
	int i;
	clock_param_t *match = NULL;

	asm volatile (
		      "ldr r1, =0x41300000;"			// Core Clock Config Reg
		      "ldr %0, [r1];"
		      "mrc p14, 0, %1, c6, c0, 0;"	// read CCLKCFG register
		      : "=r"(CCCR_reg), "=r"(CLKCFG_reg)
		      :
		      : "r1"
		      );

	clkcfg_t	=	CLKCFG_reg & 0x01;
	clkcfg_ht	=	(CLKCFG_reg & 0x04) >> 2;
	clkcfg_b	=	(CLKCFG_reg & 0x08) >> 3;

	cccr_l		=	(CCCR_reg & 0x0000001f) >> 0;
	cccr_2n		=	(CCCR_reg & 0x00000780) >> 7;
	cccr_a		=	(CCCR_reg & 0x02000000) >> 25;
#if 0
	diag_printf("clkcfg_t   = %d \n", clkcfg_t);
	diag_printf("clkcfg_ht  = %d \n", clkcfg_ht);
	diag_printf("clkcfg_b   = %d \n", clkcfg_b);
	diag_printf("cccr_l     = %d \n", cccr_l);
	diag_printf("cccr_2n    = %d \n", cccr_2n);
	diag_printf("cccr_a     = %d \n", cccr_a);
#endif
	for (i = 0; i < sizeof(pclktab) / sizeof(clock_param_t); i++) {
		if (clkcfg_t  == pclktab[i].CLKCFG_T &&
		    clkcfg_ht == pclktab[i].CLKCFG_HT &&
		    clkcfg_b  == pclktab[i].CLKCFG_B &&
		    cccr_l    == pclktab[i].CCCR_L &&
		    cccr_2n   == pclktab[i].CCCR_2N &&
		    cccr_a    == pclktab[i].CCCR_A) {
			match = &pclktab[i];
			break;
		}
	}

	if (match == NULL) {
		//diag_printf("error, unknown clock configuration\n");
	}
	return match;
}

void change_clock(unsigned char ind)
{
	clock_param_t	*cpara;
	unsigned long CCCR_reg, CLKCFG_reg;

	diag_printf("Clock speed will be changed to %d MHz", pclktab[ind].t_clock);
 	CCCR_reg = pclktab[ind].CCCR_L |
		((pclktab[ind].CCCR_2N) << 7) |
		((pclktab[ind].CCCR_A) << 25);
 	CLKCFG_reg = pclktab[ind].CLKCFG_T |
		((pclktab[ind].CLKCFG_HT) << 2) |
		((pclktab[ind].CLKCFG_B) << 3);
	asm volatile (
		      "mov	r1,	%0;"
		      "ldr	r2, =0x41300000;"			// Core Clock Config Reg
		      "ldr	r3, [r2];"					// read the register
		      "ldr	r4, =0xcc000000;"			// mask bits to preserve
		      "and	r3, r3, r4;"
		      "orr	r3, r3, r1;"				// write speed values
		      "str	r3, [r2];"				// set speed
		      "mov	r1, %1;"					// set turbo mode, set fast bus
		      "orr	r1, r1, #0x2;"				// set change frequency
		      "mov	r3, r1;"					// save value
		      "mcr	p14, 0, r1, c6, c0, 0;"		// frequency change  sequence
		      "ldr	r1, [r2];"					// dummy read from CCCR
		      "100:;"
		      "mrc	p14, 0, r1, c6, c0, 0;"		// read CLKCFG
		      "cmp r3, r1;"						// compare it with value written
		      "bne 100b;"
		      :
		      : "r"(CCCR_reg),  "r"(CLKCFG_reg)
		      : "r1", "r2", "r3", "r4"
		      );
    	cpara = get_clock_rate();
	diag_printf("\nclock speed set to %d MHz ...\n", cpara->t_clock);
}

void do_clock(int argc, char *argv[])
{
	int new_clock, ind;
	bool new_clock_set, clock_ok;
	struct option_info opts[1];
//#ifdef CYGSEM_REDBOOT_FLASH_CONFIG
//    struct config_option opt;
//#endif
	unsigned long value;
	clock_param_t *cpara, *old_clock;

	init_opts(&opts[0], 's', true, OPTION_ARG_TYPE_NUM,
		  (void *)&new_clock, (bool *)&new_clock_set, "new clock speed");
	if (!scan_opts(argc, argv, 1, opts, 1, 0, 0, "")) {
		return;
	}

	get_clock_table_index(new_clock);

	if (new_clock_set) {
		clock_ok = 1;

		old_clock = get_clock_rate();
		if (old_clock == 0) {
			diag_printf("error decoding clock settings, clock unknown\n");
			return;
		}
		ind = get_clock_table_index(new_clock);

		HAL_READ_UINT32(MDREFR, value);
		value |= 0x20000000; 				// set K0DB4
		HAL_WRITE_UINT32(MDREFR, value);

		if (pclktab[ind].t_clock > old_clock->t_clock) {
			change_core_voltage(ind);
			change_clock(ind);
		} else {
			change_clock(ind);
			change_core_voltage(ind);
		}

		if (pclktab[ind].MDREFR_K0DB2) {
			HAL_READ_UINT32(MDREFR, value);		// set K0DB2
			value |= 0x00004000;
		} else {
			HAL_READ_UINT32(MDREFR, value);		// reset K0DB0
			value &= (~0x00004000);
		}
		HAL_WRITE_UINT32(MDREFR, value);

		if (pclktab[ind].MDREFR_K0DB4) {
			HAL_READ_UINT32(MDREFR, value);		// set K0DB2
			value |= 0x20000000;
		} else {
			HAL_READ_UINT32(MDREFR, value);		// reset K0DB2
			value &= (~0x20000000);
		}
		HAL_WRITE_UINT32(MDREFR, value);
	} else {
		cpara = get_clock_rate();
		if (cpara == 0) {
			diag_printf("error decoding clock settings, clock unknown\n");
		} else {
			diag_printf("Actual Processor Clock: %d MHz   System Bus Clock %d MHz    MEM Clock %d MHz   SDRAM Clock %d MHz   LCD Clock %d MHz\n",
				    cpara->t_clock, cpara->sys_clock, cpara->mem_clock, cpara->SDRAM_clock, cpara->LCD_clock);
		}
	}
}

RedBoot_cmd("run",
            "Execute code at a location",
            "[-w <timeout>] [entry]",
            do_run
	    );

void do_run(int argc, char *argv[])
{
	typedef void code_fun(void);
	unsigned long entry;
	unsigned long oldints;
	code_fun *fun;
	bool wait_time_set;
	int  wait_time, res;
	struct option_info opts[1];
	char line[8];

	entry = entry_address;  // Default from last 'load' operation
	init_opts(&opts[0], 'w', true, OPTION_ARG_TYPE_NUM,
		  (void *)&wait_time, (bool *)&wait_time_set, "wait timeout");
	if (!scan_opts(argc, argv, 1, opts, 1, (void *)&entry, OPTION_ARG_TYPE_NUM,
		       "starting address")) {
		entry = 0xa0100000;
		diag_printf("starting run at address 0x%08X ...\n", entry);
	}
	if (wait_time_set) {
		int script_timeout_ms = wait_time * 1000;
#ifdef CYGSEM_REDBOOT_FLASH_CONFIG
		unsigned char *hold_script = script;
		script = (unsigned char *)0;
#endif
		diag_printf("About to start execution at %p - abort with ^C within %d seconds\n",
			    (void *)entry, wait_time);
		while (script_timeout_ms >= CYGNUM_REDBOOT_CLI_IDLE_TIMEOUT) {
			res = _rb_gets(line, sizeof(line), CYGNUM_REDBOOT_CLI_IDLE_TIMEOUT);
			if (res == _GETS_CTRLC) {
#ifdef CYGSEM_REDBOOT_FLASH_CONFIG
				script = hold_script;  // Re-enable script
#endif
				return;
			}
			script_timeout_ms -= CYGNUM_REDBOOT_CLI_IDLE_TIMEOUT;
		}
	}
	fun = (code_fun *)entry;
	HAL_DISABLE_INTERRUPTS(oldints);
	HAL_DCACHE_SYNC();
	HAL_ICACHE_DISABLE();
	HAL_DCACHE_DISABLE();
	HAL_DCACHE_SYNC();
	HAL_ICACHE_INVALIDATE_ALL();
	HAL_DCACHE_INVALIDATE_ALL();
#ifdef HAL_ARCH_PROGRAM_NEW_STACK
	HAL_ARCH_PROGRAM_NEW_STACK(fun);
#else
	(*fun)();
#endif
	HAL_ICACHE_ENABLE();
	HAL_DCACHE_ENABLE();
	HAL_RESTORE_INTERRUPTS(oldints);
}

cyg_int32 write_i2c_pwr(cyg_uint8 device_adr,  cyg_uint8 dat_value)
{
	cyg_uint32	value;
	unsigned int	retries;

	// write device address now

	HAL_WRITE_UINT32(PWRIDBR, (device_adr & 0xfe));
	HAL_READ_UINT32(PWRICR, value);
	value = value & ~(ICR_STOP | ICR_ALDIE);
	HAL_WRITE_UINT32(PWRICR, value | ICR_START | ICR_TB);

	HAL_READ_UINT32(PWRISR, value);
	retries = 1000000;
	while (!(value & ISR_ITE) && retries--) {	// wait for Transmit Empty
		HAL_READ_UINT32(PWRISR, value);
	}
	if (!(value & ISR_ITE)) {
		diag_printf("I2C: timeout waiting for ITE\n");
		return -1;
	}

	HAL_READ_UINT32(PWRISR, value);
	if (value & ISR_BED) {
		//diag_printf("I2C: bus error, status after write device address is: %06X\n", value);
		HAL_READ_UINT32(PWRICR, value);
		HAL_WRITE_UINT32(PWRICR, value | ICR_MA);		// send master abort
		HAL_DELAY_US(10000);
		HAL_WRITE_UINT32(PWRICR, value & ~ICR_MA);
		return -1;
	}
	HAL_WRITE_UINT32(PWRISR, ISR_ITE);	// clear ITE status

	// write data now
	HAL_WRITE_UINT32(PWRIDBR, dat_value & 0xff);
	HAL_READ_UINT32(PWRICR, value);
	value &= ~ICR_START;
	HAL_WRITE_UINT32(PWRICR, value | (ICR_TB | ICR_STOP));

	HAL_READ_UINT32(PWRISR, value);
	retries = 1000000;
	while (!(value & ISR_ITE) && retries--) {	// wait for Transmit Empty
		HAL_READ_UINT32(PWRISR, value);
	}
	if (!(value & ISR_ITE)) {
		diag_printf("I2C: timeout waiting for ITE\n");
		return -1;
	}
	HAL_READ_UINT32(PWRISR, value);
	if (value & ISR_BED) {
		//diag_printf("I2C: bus error, status after write device address is: %06X\n", value);
		HAL_READ_UINT32(PWRICR, value);
		HAL_WRITE_UINT32(PWRICR, value | ICR_MA);		// send master abort
		HAL_DELAY_US(10000);
		HAL_WRITE_UINT32(PWRICR, value & ~ICR_MA);
		return -1;
	}
	HAL_WRITE_UINT32(PWRISR, ISR_ITE);	// clear ITE status

	HAL_READ_UINT32(PWRICR, value);
	value = value & ~(ICR_STOP | ICR_START | ICR_TB);
	HAL_WRITE_UINT32(PWRICR, value);

	return 0;
}
