//==========================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):    usteinkohl
// Contributors: usteinkohl
//               Uwe Steinkohl <us@karo-electronics.de>
// Date:         2005-05-24
// Purpose:
// Description:
//
// This code is to handle the SD-Card
//
//####DESCRIPTIONEND####
//
//==========================================================================

#include <redboot.h>

#include <cyg/hal/hal_triton270.h>
#include <cyg/hal/hal_intr.h>
#include <cyg/hal/hal_cache.h>

#include <cyg/hal/hal_io.h>             // IO macros

#define DMA_ACCESS			1

#define SDCARD_DEBUG			0
#define FAT16_ONLY			0

#define MMCCLKRT_19M5			0
#define MMCCLKRT_9M75			1
#define MMCCLKRT_4M88			2
#define MMCCLKRT_2M44			3
#define MMCCLKRT_1M22			4
#define MMCCLKRT_0M609			5
#define MMCCLKRT_0M304			6

#define MMCSTAT_TIME_OUT_READ		0x00000001
#define MMCSTAT_TIME_OUT_RES		0x00000002
#define MMCSTAT_CRC_WR_ERR		0x00000004
#define MMCSTAT_CRC_RD_ERR		0x00000008
#define MMCSTAT_DAT_ERR_TOKEN		0x00000010
#define MMCSTAT_RES_CRC_ERR		0x00000020

#define MMCSTAT_CLK_EN			0x00000100
#define MMCSTAT_FLASH_ERR		0x00000200
#define MMCSTAT_SPI_WR_ERR		0x00000400
#define MMCSTAT_DATA_TRAN_DONE		0x00000800

#define MMCSTAT_PRG_DONE		0x00001000
#define MMCSTAT_END_CMD_RES		0x00002000
#define MMCSTAT_RD_STALLED		0x00004000
#define MMCSTAT_SDIO_INT		0x00008000

#define MMCSTAT_SDIO_SUSPEND_ACK	0x00010000

#define CMD_RES_TYPE_NORESP		0
#define CMD_RES_TYPE_R1			1
#define CMD_RES_TYPE_R6			1
#define CMD_RES_TYPE_R2			2
#define CMD_RES_TYPE_R3			3

#define SD_CMD0				0
#define SD_CMD2				2
#define SD_CMD3				3
#define SD_CMD4				4
#define SD_CMD7				7
#define SD_CMD9				9
#define SD_CMD10			10
#define SD_CMD12			12
#define SD_CMD13			13
#define SD_CMD15			15
#define SD_CMD16			16
#define SD_CMD17			17
#define SD_CMD18			18
#define SD_CMD55			55

#define SD_ACMD6			(6 + 128)
#define SD_ACMD41			(41 + 128)

typedef	unsigned char	u8;
typedef	unsigned short	u16;
typedef	unsigned long	u32;

typedef struct command_parameter_struct {
	unsigned char		command_number;
	unsigned short		rca;
	unsigned short		command_parameter_low;
	unsigned short		command_parameter_high;
	unsigned char		res_type;
	unsigned char		data_en;
	unsigned char		dma_en;
	unsigned char		data_write;
	unsigned char		stream_blk;
	unsigned char		busy;
	unsigned char		init;
	unsigned char		sd_4dat;
	unsigned char		*rbuffer;			// pointer to response buffer
	unsigned char		*dbuffer;			// pointer to data buffer
	int			isWinceImage;
} COMPAR;

typedef struct drive_spec_t {
	u32			partition_size;
	u32			partition_offset;
	u8			partition_type;
	u16			number_of_cylinders;
	u16			number_of_heads;
	u16			number_of_sectors_per_track;
	u16			bytes_per_sector;
	u8			sectors_per_cluster;
	u16			reserved_sectors;
	u8			number_of_copies_of_fat;
	u16			maximum_root_entries;
	u16			sectors_per_fat;
	u32			fat_start_block;
	u32			root_dir_start_block;
	u32			data_area_start_block;
	u16			number_of_files;
	int			fat16;
} drive_spec;

typedef struct dir_data_t {
	char		dir_fname[13];
	u16			dir_start_cluster;
	u32			dir_fsize;
	u16			dir_year;
	u8			dir_month;
	u8			dir_day;
	u8			dir_second;
	u8			dir_minute;
	u8			dir_hour;
} dir_data;

int	init_sdcard(COMPAR *pp);
void	enable_sdcard_power(void);
int	chk_sd_status(COMPAR *pp);
int	send_command(COMPAR *pp);
int		wait4cmd_end(void);
int	read_single_block(COMPAR *pp, unsigned long address);
void	dump_buf(unsigned char *address, unsigned short length);
int	get_dir_data(drive_spec *ds, COMPAR *pp);
void	dump_dir(dir_data *dird, drive_spec *ds);
int	get_dir_entry(dir_data	*dentry, drive_spec *ds, COMPAR *pp, unsigned char *filename);
void	kill_space(char *string);
int	read_multiple_block(COMPAR *pp, unsigned long address, unsigned long blocks);

int	read_file(dir_data *dird, drive_spec *ds, unsigned long *address, COMPAR *pp);
int	get_next_cluster(dir_data *dird, drive_spec *ds,  COMPAR *pp, unsigned short last_cluster);

int	isprint(int character);

RedBoot_cmd("sdcard",
	    "read from SD-Card",
	    "[-d] [-f <filename>] [-b <base_address]",
	    do_sdcard
	    );

static inline unsigned long cmd_value(COMPAR *pp)
{
	unsigned long cvalue;

	cvalue = (((pp->res_type & 0x3) << 0) |
		  ((pp->data_en & 0x1) << 2) |
		  ((pp->data_write & 0x1) << 3) |
		  ((pp->stream_blk & 0x1) << 4) |
		  ((pp->busy & 0x1) << 5) |
		  ((pp->init & 0x1) << 6) |
		  ((pp->dma_en & 0x1) << 7) |
		  ((pp->sd_4dat & 0x1) << 8));
	return cvalue;
}

void do_sdcard(int argc, char *argv[])
{
	struct option_info opts[3];
	bool dflag_set, base_address_set, filename_set;
	unsigned char *fn_buffer;
	unsigned long base_address;
	int dflag;
	static COMPAR	cpar;
	static unsigned char	rbuffer[32];
	static unsigned char	dbuffer[512];
	drive_spec	my_drive;
	drive_spec	*ds = &my_drive;
	u16		*p16 = (u16 *)dbuffer;
	int		valid_partition;
	dir_data	dentry;
	int i;

	init_opts(&opts[0], 'd', false, OPTION_ARG_TYPE_FLG, (void *)&dflag,
		  &dflag_set, "directory");
	init_opts(&opts[1], 'b', true, OPTION_ARG_TYPE_NUM, (void *)&base_address,
		  &base_address_set, "base address to load file contents");
	init_opts(&opts[2], 'f', true, OPTION_ARG_TYPE_STR, (void *)&fn_buffer,
		  &filename_set, "filename");

	if (!scan_opts(argc, argv, 1, opts, 3, 0, 0, "")) {
		return;
	}
	if (dflag & base_address_set) {
		diag_printf("warning, -b parameter is ignored\n");
	}

	if (dflag & filename_set) {
		diag_printf("warning, -f parameter is ignored\n");
	}

#ifdef CYGBLD_BUILD_REDBOOT_WITH_WINCE_SUPPORT
	// Do not check for the base address now since
	// possibly we are about to load a wince .BIN image.
	// We will validate it latter instead.

	if (!dflag && !filename_set) {
		diag_printf("error, filename must be specified with -f parameter\n");
		return;
	}

	if (!base_address_set) {
		base_address = 0;
	}
#else
	if (base_address_set & !filename_set) {
		diag_printf("error, filename must be specified with -f parameter\n");
		return;
	}

	if (!base_address_set & filename_set) {
		diag_printf("error, base address must be specified with -b parameter\n");
		return;
	}
#endif
	cpar.rbuffer = rbuffer;		// specify buffer for command response
	cpar.dbuffer = dbuffer;		// specify buffer for data transfer

	if (dflag) {
		if (chk_sd_status(&cpar) < 0) {
			return;
		}
#if SDCARD_DEBUG
		diag_printf("\n\nreading master boot record now\n");
#endif
		if (read_single_block(&cpar, 0) < 0) {
			diag_printf("read single block command failed!\n");
			return;
		}
#if SDCARD_DEBUG
		diag_printf("\n\n");
		dump_buf(dbuffer, 512);
#endif
		if (*(p16 + 255) != 0xaa55) {
			diag_printf("warning: no patition table found on drive\n");
			diag_printf("exiting command\n");
			return;
		}

		//search for first FAT16 partition
		valid_partition = 0;
		for (i = 0; i < 4; i++) {
			ds->partition_size = dbuffer[446 + i * 16 + 12] +
				(dbuffer[446 + i * 16 + 13] << 8) +
				(dbuffer[446 + i * 16 + 14] << 16) +
				(dbuffer[446 + i * 16 + 15] << 24);

			ds->partition_offset = dbuffer[446 + i * 16 + 8] +
				(dbuffer[446 + i * 16 + 9] << 8) +
				(dbuffer[446 + i * 16 + 10] << 16) +
				(dbuffer[446 + i * 16 + 11] << 24);

			ds->partition_type = dbuffer[446 + i * 16 + 4];

			if ((ds->partition_type != 0x01 &&
			     ds->partition_type != 0x04 &&
			     ds->partition_type != 0x05 &&
			     ds->partition_type != 0x06) ||
			    ds->partition_size == 0 || ds->partition_offset == 0) {
				continue;
			} else {
				valid_partition = i + 1;
				break;
			}
		}

		if (!valid_partition) {
			diag_printf("warning, no valid partiton table found\n");
			ds->partition_offset = 0;
		} else {
#if SDCARD_DEBUG
			diag_printf("Partition Size: 0x%08X\n", ds->partition_size);
			diag_printf("Partition Offset: 0x%08X\n", ds->partition_offset);
			diag_printf("Partition Type: 0x%08X\n", ds->partition_type);
#endif
		}

		if (read_single_block(&cpar, ds->partition_offset * 512) < 0) {
			diag_printf("read single block command failed!\n");
			return;
		}
#if SDCARD_DEBUG
		diag_printf("\n\nreading FAT boot record now:\n");
		dump_buf(dbuffer, 512);
#endif
#if !FAT16_ONLY
		if (dbuffer[0x36] =='F' &&
		    dbuffer[0x37] == 'A' &&
		    dbuffer[0x38] == 'T' &&
		    dbuffer[0x39] == '1' &&
		    (dbuffer[0x3A] == '2' || dbuffer[0x3A] == '6')) {
			if (dbuffer[0x3A] == '6') {
#if SDCARD_DEBUG
				diag_printf("FAT16 found\n");
#endif
				ds->fat16 = 1;
			} else {
				diag_printf("FAT12 found\n");
				ds->fat16 = 0;
			}
		} else {
			diag_printf("no valid FAT entry found\n");
			while (1);
		}
#else
		ds->fat16 = 1;
#endif
		ds->bytes_per_sector = dbuffer[0x0b] + (dbuffer[0x0c] << 8);
		ds->sectors_per_cluster = dbuffer[0x0d];
		ds->reserved_sectors = dbuffer[0x0e] + (dbuffer[0x0f] << 8);
		ds->number_of_copies_of_fat = dbuffer[0x10];
		ds->maximum_root_entries = dbuffer[0x11] + (dbuffer[0x12] << 8);
		ds->sectors_per_fat = dbuffer[0x16] + (dbuffer[0x17] << 8);

		ds->fat_start_block = ds->partition_offset + ds->reserved_sectors;
		ds->root_dir_start_block = ds->fat_start_block + ds->sectors_per_fat * ds->number_of_copies_of_fat;
		ds->data_area_start_block = ds->root_dir_start_block + ds->maximum_root_entries * 32 / ds->bytes_per_sector;
		ds->number_of_files = 0;
#if SDCARD_DEBUG
		diag_printf("partition size is: .................... %lu blocks\n",
			    ds->partition_size);
		diag_printf("partition offset is: .................. %lu blocks\n",
			    ds->partition_offset);

		diag_printf("bytes per sector ...................... %d\n",
			    ds->bytes_per_sector);
		diag_printf("sectors per cluster ................... %d\n",
			    ds->sectors_per_cluster);
		diag_printf("reserved sectors found ................ %d\n",
			    ds->reserved_sectors);
		diag_printf("number of FAT copies .................. %d\n",
			    ds->number_of_copies_of_fat);
		diag_printf("maximum entries in root file system ... %d\n",
			    ds->maximum_root_entries);
		diag_printf("sectors per fat ....................... %d\n",
			    ds->sectors_per_fat);

		diag_printf("FAT starts at block ................... %lu\n",
			    ds->fat_start_block);
		diag_printf("Root DIR starts at block .............. %lu\n",
			    ds->root_dir_start_block);
		diag_printf("Data (Cluster 2) starts at block ...... %lu\n",
			    ds->data_area_start_block);
#endif
		if (get_dir_data(ds, &cpar) < 0) {
			diag_printf("failed to read directory entries !\n");
			return;
		}
		diag_printf("\n\n%d files found in directory\n", ds->number_of_files);
	}

	if (filename_set) {
		if (chk_sd_status(&cpar) < 0) {
			return;
		}
#if SDCARD_DEBUG
		diag_printf("\n\nreading master boot record now\n");
#endif
		if (read_single_block(&cpar, 0) < 0) {
			diag_printf("read single block command failed!\n");
			return;
		}
#if SDCARD_DEBUG
		diag_printf("\n\n");
		dump_buf(dbuffer, 512);
#endif
		if (*(p16 + 255) != 0xaa55) {
			diag_printf("warning: no patition table found on drive\n");
			return;
		}

		//search for first FAT16 partition
		valid_partition = 0;
		for (i = 0; i < 4; i++) {
			ds->partition_size = dbuffer[446 + i * 16 + 12] + (dbuffer[446 + i * 16 + 13] << 8)
				+ (dbuffer[446 + i * 16 + 14] << 16) + (dbuffer[446 + i * 16 + 15] << 24);

			ds->partition_offset = dbuffer[446 + i * 16 + 8] + (dbuffer[446 + i * 16 + 9] << 8)
				+ (dbuffer[446 + i * 16 + 10] << 16) + (dbuffer[446 + i * 16 + 11] << 24);

			ds->partition_type = dbuffer[446 + i * 16 + 4];

			if ((ds->partition_type != 0x01 &&
			     ds->partition_type != 0x04 &&
			     ds->partition_type != 0x05 &&
			     ds->partition_type != 0x06) ||
			    ds->partition_size == 0 || ds->partition_offset == 0) {
				continue;
			} else {
				valid_partition = i + 1;
				break;
			}
		}
		if (!valid_partition) {
			diag_printf("warning, no valid partiton table found\n");
			ds->partition_offset = 0;
		}
		if (read_single_block(&cpar, ds->partition_offset * 512) < 0) {
			diag_printf("read single block command failed!\n");
			return;
		}
#if SDCARD_DEBUG
		diag_printf("\n\nreading FAT boot record now:\n");
		dump_buf(dbuffer, 512);
#endif
#if !FAT16_ONLY
		if (dbuffer[0x36] == 'F' &&
		    dbuffer[0x37] == 'A' &&
		    dbuffer[0x38] == 'T' &&
		    dbuffer[0x39] == '1' &&
		    (dbuffer[0x3A] == '2' || dbuffer[0x3A] == '6')) {
			if (dbuffer[0x3A] == '6') {
#if SDCARD_DEBUG
				diag_printf("FAT16 found\n");
#endif
				ds->fat16 = 1;
			} else {
				diag_printf("FAT12 found\n");
				ds->fat16 = 0;
			}
		} else {
			diag_printf("no valid FAT entry found\n");
			while (1);
		}
#else
		ds->fat16 = 1;
#endif
		ds->bytes_per_sector = dbuffer[0x0b] + (dbuffer[0x0c] << 8);
		ds->sectors_per_cluster = dbuffer[0x0d];
		ds->reserved_sectors = dbuffer[0x0e] + (dbuffer[0x0f] << 8);
		ds->number_of_copies_of_fat = dbuffer[0x10];
		ds->maximum_root_entries = dbuffer[0x11] + (dbuffer[0x12] << 8);
		ds->sectors_per_fat = dbuffer[0x16] + (dbuffer[0x17] << 8);

		ds->fat_start_block = ds->partition_offset + ds->reserved_sectors;
		ds->root_dir_start_block = ds->fat_start_block + ds->sectors_per_fat * ds->number_of_copies_of_fat;
		ds->data_area_start_block = ds->root_dir_start_block + ds->maximum_root_entries * 32 / ds->bytes_per_sector;
		ds->number_of_files = 0;
#if SDCARD_DEBUG
		diag_printf("partition size is: .................... %lu blocks\n",
			    ds->partition_size);
		diag_printf("partition offset is: .................. %lu blocks\n",
			    ds->partition_offset);

		diag_printf("bytes per sector ...................... %d\n",
			    ds->bytes_per_sector);
		diag_printf("sectors per cluster ................... %d\n",
			    ds->sectors_per_cluster);
		diag_printf("reserved sectors found ................ %d\n",
			    ds->reserved_sectors);
		diag_printf("number of FAT copies .................. %d\n",
			    ds->number_of_copies_of_fat);
		diag_printf("maximum entries in root file system ... %d\n",
			    ds->maximum_root_entries);
		diag_printf("sectors per fat ....................... %d\n",
			    ds->sectors_per_fat);

		diag_printf("FAT starts at block ................... %lu\n",
			    ds->fat_start_block);
		diag_printf("Root DIR starts at block .............. %lu\n",
			    ds->root_dir_start_block);
		diag_printf("Data (Cluster 2) starts at block ...... %lu\n",
			    ds->data_area_start_block);
#endif
		if (get_dir_entry(&dentry, ds, &cpar, fn_buffer) < 0) {
			diag_printf("file %s not found !\n", dentry.dir_fname);
			return;
		}

		if (read_file(&dentry, ds, (unsigned long *)base_address, &cpar) < 0) {
			diag_printf("failed to read file %s\n", dentry.dir_fname);
			return;
		}
#ifdef CYGBLD_BUILD_REDBOOT_WITH_WINCE_SUPPORT
		if (!cpar.isWinceImage) {
			diag_printf("\n%s loaded       address: 0x%08X      size: %10lu bytes\n\n",
				    dentry.dir_fname, base_address, dentry.dir_fsize);
		}
#else
		diag_printf("\n%s loaded       address: 0x%08X      size: %10lu bytes\n\n",
			    dentry.dir_fname, base_address, dentry.dir_fsize);
#endif
		return;
	}
}

int chk_sd_status(COMPAR *pp)
{
	unsigned char	rbuffer[32];
#if SDCARD_DEBUG
	int i;
#endif

	pp->command_number = SD_CMD13;
	if (send_command(pp) < 0) {
		diag_printf("initialising SD-Card interface\n");
		if (init_sdcard(pp) < 0) {
			diag_printf("card initialisation failed !\n");
			return -1;
		} else {
			return 0;
		}
	}
#if SDCARD_DEBUG
	diag_printf("Response got from CMD13: ");
	for (i = 0; i < 6; i++) {
		diag_printf("%02X ", rbuffer[i]);
	}
	diag_printf("\n\n\n");
#endif
	if (rbuffer[3] != 0x09 && (rbuffer[2] & 0x01) != 0x00) {
		diag_printf("warning, Card is not in the Transfer state as expected !\n");
		diag_printf("initialising SD-Card interface\n");
		if (init_sdcard(pp) < 0) {
			diag_printf("card initialisation failed !\n");
			return -1;
		} else {
			return 0;
		}
	}
	return 0;

}

int init_sdcard(COMPAR *pp)
{
	unsigned long	value;
	int		timeout;
	unsigned char	tbuffer[6];
	unsigned short	newrca;
	unsigned long	trans_speed;
	unsigned long	read_bl_len;
	unsigned long	c_size;
	unsigned long	c_size_mult;
	unsigned long	sector_size;
	unsigned long	conbuf[3];
	unsigned long	mult;
	unsigned long	blen;
	unsigned long	card_size;
	unsigned long	trans_rate;
	int		i;
	unsigned long	trans_rate_exp[8] = {100, 1000, 10000, 100000, 0, 0, 0, 0};
	unsigned short	time_mantissa[16] = {0, 10, 12, 13, 15, 20, 25, 30, 35, 40, 45, 50, 55, 60, 70, 80};

	enable_sdcard_power();

	pp->rca = 0;			// relative card address

	HAL_READ_UINT32(ICMR, value);
	value &= (~0x00800000);
	HAL_WRITE_UINT32(ICMR, value);	// mask interrupt source SD-card controller

	HAL_WRITE_UINT32(MMCCLKRT, MMCCLKRT_0M304);	// set clock rate to lowest rate first
	HAL_WRITE_UINT32(MMCSPI, 0);	// do not use SPI mode
	HAL_WRITE_UINT32(MMCRESTO, 0x7f); // set to maximum timeout
	HAL_WRITE_UINT32(MMCBLKLEN, 512);
	HAL_WRITE_UINT32(MMCNOB, 1);

	// start the clock now
	HAL_WRITE_UINT32(MMCSTRPCL, 2);	// start MMCLK
	HAL_READ_UINT32(MMCSTRPCL, value);
	while (value & 2) {		// wait until clock is started
		HAL_DELAY_US(1000);
		HAL_READ_UINT32(MMCSTRPCL, value);
	}
	HAL_DELAY_US(100 * 1000);

	pp->command_number = SD_CMD0;
	if (send_command(pp) < 0) {
		diag_printf("CMD0 failed, repeating command \n");
		return -1;
	}

	timeout = 100;
	pp->command_number = SD_ACMD41;
	pp->command_parameter_high = 0x001f;	// 2,8V - 3,3V

	while (timeout > 0) {
		if (send_command(pp) < 0) {
			HAL_DELAY_US(1000);
			timeout--;
			continue;
		}
		if (!(pp->rbuffer[1] & 0x80)) {  // card not powered up
			HAL_DELAY_US(1000);
			timeout--;
			continue;
		} else {
			break;
		}
	}
	if (timeout == 0) {
		diag_printf("failed to power up card, ACMD41 failed\n");
		return -1;
	}
#if SDCARD_DEBUG
	diag_printf("Response got from ACMD41: ");
	for (i = 0; i < 6; i++) {
		diag_printf("%02X ", pp->rbuffer[i]);
	}
	diag_printf("\n\n\n");
#endif
	pp->command_number = SD_CMD2;
	if (send_command(pp) < 0) {
		diag_printf("CMD2 failed !!!\n");
		return -1;
	}
#if SDCARD_DEBUG
	diag_printf("Response got from CMD2: ");
	for (i = 0; i < 17; i++) {
		diag_printf("%02X ", pp->rbuffer[i]);
	}
	diag_printf("\n\n\n");
#endif
	for (i = 0; i < 5; i++) {
		tbuffer[i] = pp->rbuffer[i + 4];
	}
	tbuffer[5] = '\0';

	diag_printf("found SD-Card: %s\n", tbuffer);

	pp->command_number = SD_CMD3;
	if (send_command(pp) < 0) {
		diag_printf("CMD3 failed !!!\n");
		return -1;
	}
#if SDCARD_DEBUG
	diag_printf("Response got from CMD3: ");
	for (i = 0; i < 6; i++) {
		diag_printf("%02X ", pp->rbuffer[i]);
	}
	diag_printf("\n\n\n");
#endif
	newrca = 0;
	newrca = (pp->rbuffer[1] << 8) | pp->rbuffer[2];
#if 1
	diag_printf("new RCA is: %04X\n", newrca);
#endif
	pp->rca = newrca;			// relative card address

//////////////////////////////////////////////////////////////////////////////////
// SEND_CSD Command
//
// get clock information of CSD register
//
//////////////////////////////////////////////////////////////////////////////////

	pp->command_number = SD_CMD9;
	if (send_command(pp) < 0) {
		diag_printf("CMD9 (SEND_CSD) failed !!!\n");
		return -1;
	}
#if SDCARD_DEBUG
	diag_printf("Response got from CMD9 (SEND_CSD): ");
	for (i = 0; i < 17; i++) {
		diag_printf("%02X ", pp->rbuffer[i]);
	}
	diag_printf("\n\n\n");
#endif
	trans_speed = pp->rbuffer[4];
	read_bl_len = pp->rbuffer[6] & 0x0f;
	conbuf[0] = (unsigned long)(pp->rbuffer[9] & 0xc0);
	conbuf[1] = (unsigned long)(pp->rbuffer[8] & 0xff);
	conbuf[2] = (unsigned long)(pp->rbuffer[7] & 0x03);
	c_size = (conbuf[0] >> 6) | (conbuf[1] << 2)  | (conbuf[2] << 10);
	conbuf[0] = (pp->rbuffer[11] & 0x80);
	conbuf[1] = (pp->rbuffer[10] & 0x03);
	c_size_mult = conbuf[0] >> 7 | conbuf[1] << 1;
	conbuf[0] = (pp->rbuffer[12] & 0x80);
	conbuf[1] = (pp->rbuffer[11] & 0x3f);
	sector_size = conbuf[0] >> 7 | conbuf[1] << 1;

	for (i = 0, mult = 4; i < c_size_mult; i++)
		mult *= 2;

	for (i = 0, blen = 1; i < read_bl_len; i++)
		blen *= 2;

	card_size = (c_size + 1) * mult * blen;
	diag_printf("total card capacity is %lu.%03lu MBytes\n",
		    card_size / 1024 / 1024, card_size / 1024 % 1024);

	conbuf[0] = (unsigned long)trans_speed & 0x07;
	conbuf[1] = ((unsigned long)(trans_speed & 0x78)) >> 3;

	trans_rate = time_mantissa[conbuf[1]] *	trans_rate_exp[conbuf[0]] / 10;	// in kbit/sec

	diag_printf("transfer rate of card is %lu kbit/sec\n", trans_rate);

	if (trans_rate >= 19500) {
		diag_printf("set interface clock rate to 19,5 MHz\n\n");
		HAL_WRITE_UINT32(MMCCLKRT, MMCCLKRT_19M5);
	} else if (trans_rate >= 9750) {
		diag_printf("set interface clock rate to 9,75 MHz\n\n");
		HAL_WRITE_UINT32(MMCCLKRT, MMCCLKRT_9M75);
	} else if (trans_rate >= 4880) {
		diag_printf("set interface clock rate to 4,88 MHz\n\n");
		HAL_WRITE_UINT32(MMCCLKRT, MMCCLKRT_4M88);
	} else if (trans_rate >= 2440) {
		diag_printf("set interface clock rate to 2,44 MHz\n\n");
		HAL_WRITE_UINT32(MMCCLKRT, MMCCLKRT_2M44);
	} else if (trans_rate >= 1220) {
		diag_printf("set interface clock rate to 1,22 MHz\n\n");
		HAL_WRITE_UINT32(MMCCLKRT, MMCCLKRT_1M22);
	} else if (trans_rate >= 609) {
		diag_printf("set interface clock rate to 609 kHz\n\n");
		HAL_WRITE_UINT32(MMCCLKRT, MMCCLKRT_0M609);
	}
/////////////////////////////////////////////////////////////////////////////////
	pp->command_number = SD_CMD7;
	if (send_command(pp) < 0) {
		diag_printf("CMD7 failed !!!\n");
		return -1;
	}
#if SDCARD_DEBUG
	diag_printf("Response got from CMD7: ");
	for (i = 0; i < 6; i++) {
		diag_printf("%02X ", pp->rbuffer[i]);
	}
	diag_printf("\n\n\n");
#endif
	pp->command_number = SD_CMD13;
	if (send_command(pp) < 0) {
		diag_printf("CMD13 failed !!!\n");
		return -1;
	}
#if SDCARD_DEBUG
	diag_printf("Response got from CMD13: ");
	for (i = 0; i < 6; i++) {
		diag_printf("%02X ", pp->rbuffer[i]);
	}
	diag_printf("\n\n\n");
#endif
	if (pp->rbuffer[3] != 0x09 && (pp->rbuffer[2] & 0x01) != 0x00) {
		diag_printf("error, Card is not in the Transfer state as expected !\n");
		return -1;
	}

	pp->command_number = SD_ACMD6;
	if (send_command(pp) < 0) {
		diag_printf("ACMD6 failed !!!\n");
		return -1;
	}

	return 1;

}

int send_command(COMPAR *pp)
{
	unsigned long cvalue;
	int i;
	unsigned short val;
	unsigned char	*cp;
	unsigned char	cnumber;

	//diag_printf("processing CMD: %d\n", pp->command_number);
	cnumber = pp->command_number;
	switch (pp->command_number) {
		case SD_CMD0:
			pp->command_parameter_low = 0;	// stuff bits
			pp->command_parameter_high = 0;	// RCA Relative Card Address;
			pp->res_type = CMD_RES_TYPE_NORESP;
			pp->data_en = 0;
			pp->dma_en = 0;
			pp->data_write = 0;
			pp->stream_blk = 0;
			pp->busy = 0;
			pp->init = 0;
			pp->sd_4dat = 1;
			break;
		case SD_CMD2:
			pp->command_parameter_low = 0;
			pp->command_parameter_high = 0;
			pp->res_type = CMD_RES_TYPE_R2;
			pp->data_en = 0;
			pp->dma_en = 0;
			pp->data_write = 0;
			pp->stream_blk = 0;
			pp->busy = 0;
			pp->init = 0;
			pp->sd_4dat = 1;
			break;
		case SD_CMD3:
			pp->command_parameter_low = 0;	// stuff bits
			pp->command_parameter_high = 0;	// stuff bits
			pp->res_type = CMD_RES_TYPE_R6;
			pp->data_en = 0;
			pp->dma_en = 0;
			pp->data_write = 0;
			pp->stream_blk = 0;
			pp->busy = 0;
			pp->init = 0;
			pp->sd_4dat = 1;
			break;
		case SD_CMD7:
			pp->command_parameter_low = 0;
			pp->command_parameter_high = pp->rca;
			pp->res_type = CMD_RES_TYPE_R1;
			pp->dma_en = 0;
			pp->data_en = 0;
			pp->data_write = 0;
			pp->stream_blk = 0;
			pp->busy = 0;
			pp->init = 0;
			pp->sd_4dat = 1;
			break;
		case SD_CMD9:		// SEND_CSD
			pp->command_parameter_low = 0;
			pp->command_parameter_high = pp->rca;
			pp->res_type = CMD_RES_TYPE_R2;
			pp->dma_en = 0;
			pp->data_en = 0;
			pp->data_write = 0;
			pp->stream_blk = 0;
			pp->busy = 0;
			pp->init = 0;
			pp->sd_4dat = 1;
			break;
		case SD_CMD12:		// Stop Transmission
			pp->command_parameter_low = 0;
			pp->command_parameter_high = 0;
			pp->res_type = CMD_RES_TYPE_R1;
			pp->dma_en = 0;
			pp->data_en = 0;
			pp->data_write = 0;
			pp->stream_blk = 0;
			pp->busy = 0;
			pp->init = 0;
			pp->sd_4dat = 1;
			break;
		case SD_CMD13:
			pp->command_parameter_low = 0;
			pp->command_parameter_high = pp->rca;
			pp->res_type = CMD_RES_TYPE_R1;
			pp->dma_en = 0;
			pp->data_en = 0;
			pp->data_write = 0;
			pp->stream_blk = 0;
			pp->busy = 0;
			pp->init = 0;
			pp->sd_4dat = 1;
			break;
		case SD_CMD17:		// READ_SINGLE_BLOCK
			pp->res_type = CMD_RES_TYPE_R1;
			pp->dma_en = 0;
			pp->data_en = 1;
			pp->data_write = 0;
			pp->stream_blk = 0;
			pp->busy = 0;
			pp->init = 0;
			pp->sd_4dat = 1;
			break;

		case SD_ACMD6:
			// we issue a CMD 55 first.
			pp->command_parameter_low = 0;	// stuff bits
			pp->command_parameter_high = 0;	// RCA Relative Card Address;
			pp->res_type = CMD_RES_TYPE_R1;
			pp->dma_en = 0;
			pp->data_en = 0;
			pp->data_write = 0;
			pp->stream_blk = 0;
			pp->busy = 0;
			pp->init = 0;
			pp->sd_4dat = 1;
			HAL_WRITE_UINT32(MMCCMD, 55);	// command number
			HAL_WRITE_UINT32(MMCARGH, pp->rca);
			HAL_WRITE_UINT32(MMCARGL, pp->command_parameter_low);

			cvalue = cmd_value(pp);
			HAL_WRITE_UINT32(MMCCMDAT, cvalue);
			if (wait4cmd_end() < 0) {
				diag_printf("CMD55 failed !!!\n");
				return -1;
			}

			// issue CMD 6 now
			pp->command_parameter_low = 2;	// 4 Bit Bus width
			pp->command_parameter_high = 0;
			pp->res_type = CMD_RES_TYPE_R1;
			pp->dma_en = 0;
			pp->data_en = 0;
			pp->data_write = 0;
			pp->stream_blk = 0;
			pp->busy = 1;
			pp->init = 0;
			pp->sd_4dat = 1;
			cnumber = 6;
			break;

		case SD_ACMD41:
			// we issue a CMD 55 first.
			pp->command_parameter_low = 0;	// stuff bits
			pp->command_parameter_high = 0;	// RCA Relative Card Address;
			pp->res_type = CMD_RES_TYPE_R1;
			pp->dma_en = 0;
			pp->data_en = 0;
			pp->data_write = 0;
			pp->stream_blk = 0;
			pp->busy = 0;
			pp->init = 0;
			pp->sd_4dat = 1;
			HAL_WRITE_UINT32(MMCCMD, 55);	// command number
			HAL_WRITE_UINT32(MMCARGH, pp->rca);
			HAL_WRITE_UINT32(MMCARGL, pp->command_parameter_low);

			cvalue = cmd_value(pp);
			HAL_WRITE_UINT32(MMCCMDAT, cvalue);
			if (wait4cmd_end() < 0) {
				diag_printf("CMD55 failed !!!\n");
				return -1;
			}

			// issue CMD 41 now
			pp->command_parameter_low = 0;
			pp->command_parameter_high = 0x001f;	// 2,8V - 3,3V
			pp->res_type = CMD_RES_TYPE_R3;
			pp->dma_en = 0;
			pp->data_en = 0;
			pp->data_write = 0;
			pp->stream_blk = 0;
			pp->busy = 1;
			pp->init = 0;
			pp->sd_4dat = 1;
			cnumber = 41;
			break;
		default:
#if SDCARD_DEBUG
			diag_printf("found unsupported command: %d\n", pp->command_number);
#else
			diag_printf("found unsupported command\n\n");
#endif
			return -1;
	}

	HAL_WRITE_UINT32(MMCCMD, pp->command_number);	// command number
	HAL_WRITE_UINT32(MMCARGH, pp->command_parameter_high);	// command parameter upper 16 bits
	HAL_WRITE_UINT32(MMCARGL, pp->command_parameter_low);	// command parameter lower 16 bits

	cvalue = cmd_value(pp);
	//diag_printf("writing into command register: 0x%08X\n", cvalue);
	HAL_WRITE_UINT32(MMCCMDAT, cvalue);

	if (wait4cmd_end() < 0) {
		return -1;
	}

	if (pp->res_type == CMD_RES_TYPE_NORESP) {
		return 0;
	}

	cp = pp->rbuffer;
	if (pp->res_type == CMD_RES_TYPE_R2) {
		for (i = 0; i < 17 / 2; i++) {
			HAL_READ_UINT16(MMCRES, val);
			*(cp++) = (unsigned char)((val >> 8) & 0xff);
			*(cp++) = (unsigned char)(val & 0xff);
		}
	} else {
		for (i = 0; i < 6 / 2; i++) {
			HAL_READ_UINT16(MMCRES, val);
			*(cp++) = (unsigned char)((val >> 8) & 0xff);
			*(cp++) = (unsigned char)(val & 0xff);
		}
	}
	return 0;
}

int wait4cmd_end(void)
{
	int timeout;
	unsigned long cvalue;
	unsigned long val;

	timeout = 1000;
	HAL_READ_UINT32(MMCSTAT, val);
	while (!(val & MMCSTAT_END_CMD_RES)) {
		HAL_DELAY_US(1000);
		timeout--;
		if (timeout <= 0){
			break;
		}
		HAL_READ_UINT32(MMCSTAT, val);
	}
	if (timeout <= 0) {
		diag_printf("Command timed out, no response from card controller\n");
		//HAL_READ_UINT32(MMCSTAT, val);
		//diag_printf("Status register: 0x%08X\n", val);
		return -1;
	}

	HAL_READ_UINT32(MMCSTAT, cvalue);

	if (cvalue & MMCSTAT_TIME_OUT_RES) {
		diag_printf("Command Response Time Out Error\n");
		return -1;
	}

	if (cvalue & MMCSTAT_RES_CRC_ERR) {
		diag_printf("Response CRC Error occured\n");
		return -1;
	}
	return 0;
}

#if DMA_ACCESS			// DMA Version
int read_single_block(COMPAR *pp, unsigned long address)
{
	unsigned long val, cvalue;

	HAL_DCACHE_SYNC();
	HAL_WRITE_UINT32(MMCNOB, 1);

	HAL_WRITE_UINT32(DCSR8, 0x40000000);	// set DMA channel to no descriptor fetch mode

	HAL_WRITE_UINT32(DSADR8, 0x41100040);	// SDCARD FIFO
	HAL_WRITE_UINT32(DTADR8, (unsigned long)pp->dbuffer); // target address
	HAL_WRITE_UINT32(DCMD8, 0x60034200);	// command
	HAL_WRITE_UINT32(DCSR8, 0xc0000000);	// run channel

	pp->command_parameter_low = address & 0xffff;
	pp->command_parameter_high = (address & 0xffff0000) >> 16;

	pp->command_number = SD_CMD17;
	pp->res_type = CMD_RES_TYPE_R1;
	pp->data_en = 1;
	pp->dma_en = 1;
	pp->data_write = 0;
	pp->stream_blk = 0;
	pp->busy = 0;
	pp->init = 0;
	pp->sd_4dat = 1;

	HAL_WRITE_UINT32(MMCCMD, pp->command_number);		// command number
	HAL_WRITE_UINT32(MMCARGH, pp->command_parameter_high);	// command parameter upper 16 bits
	HAL_WRITE_UINT32(MMCARGL, pp->command_parameter_low);	// command parameter lower 16 bits

	cvalue = cmd_value(pp);
	HAL_WRITE_UINT32(MMCCMDAT, cvalue);

	HAL_READ_UINT32(DCMD8, val);		// command
	while (val & 0x1fff) {
		HAL_READ_UINT32(DCMD8, val);
	}

	HAL_WRITE_UINT32(DCSR8, 0x40000000);	// set DMA channel to no descriptor fetch mode
#if 0
	HAL_READ_UINT32(MMCIREG, val);
	while (!(val & 0x1)) {	// wait for DATA_TRANS_DONE

		HAL_READ_UINT32(MMCIREG, val);
	}
#endif
	//dump_buf((unsigned char *)pp->dbuffer, 512);

	if (wait4cmd_end() < 0) {
		diag_printf("read single block command timed out !!!!");
		return -1;
	}
#if 0
	HAL_READ_UINT32(DSADR8, val);
	diag_printf("DMA source address: 0x%08X\n", val);
	HAL_READ_UINT32(DTADR8, val);	// target address
	diag_printf("DMA target address: 0x%08X\n", val);

	HAL_READ_UINT32(DCMD8, val);	// command
	diag_printf("DMA command register: 0x%08X\n", val);

	HAL_READ_UINT32(DCSR8, val);
	diag_printf("DMA status register: 0x%08X\n", val);
#endif
	return 0;

}
#else		// no DMA
int read_single_block(COMPAR *pp, unsigned long address)
{
	int length;
	int i;
	int bp;
	unsigned long val, cvalue;
	unsigned char val8;
	unsigned long	*bap;

	HAL_WRITE_UINT32(MMCNOB, 1);

	bap = (unsigned long *)pp->dbuffer;

	pp->command_parameter_low = address & 0xffff;
	pp->command_parameter_high = (address & 0xffff0000) >> 16;

	pp->command_number = SD_CMD17;
	pp->res_type = CMD_RES_TYPE_R1;
	pp->data_en = 1;
	pp->dma_en = 0;
	pp->data_write = 0;
	pp->stream_blk = 0;
	pp->busy = 0;
	pp->init = 0;
	pp->sd_4dat = 1;

	HAL_WRITE_UINT32(MMCCMD, pp->command_number);	// command number
	HAL_WRITE_UINT32(MMCARGH, pp->command_parameter_high);	// command parameter upper 16 bits
	HAL_WRITE_UINT32(MMCARGL, pp->command_parameter_low);	// command parameter lower 16 bits

	cvalue = cmd_value(pp);
	HAL_WRITE_UINT32(MMCCMDAT, cvalue);

	length = 512;
	bp = 0;

	while (length > 0) {
		HAL_READ_UINT32(MMCIREG, val);
		while (!(val & 0x20)) {	// wait for RXFIFO_RD_REQ
			//HAL_DELAY_US(100);

			HAL_READ_UINT32(MMCIREG, val);
		}
#if 0
		for (i = 0; i < 32; i++) {
			HAL_READ_UINT8(MMCRXFIFO, val8);
			pp->dbuffer[bp++] = val8;
		}
#else
		*(bap++) = *(unsigned long *)MMCRXFIFO;
		*(bap++) = *(unsigned long *)MMCRXFIFO;
		*(bap++) = *(unsigned long *)MMCRXFIFO;
		*(bap++) = *(unsigned long *)MMCRXFIFO;
		*(bap++) = *(unsigned long *)MMCRXFIFO;
		*(bap++) = *(unsigned long *)MMCRXFIFO;
		*(bap++) = *(unsigned long *)MMCRXFIFO;
		*(bap++) = *(unsigned long *)MMCRXFIFO;
#endif
		length -=32;
	}
	if (wait4cmd_end() < 0) {
		diag_printf("read single block command timed out !!!!");
		return -1;
	}

	return 0;

}
#endif

#if 1
void dump_buf(unsigned char *address, u16 length)
{
	u16 lines = length / 16;
	u16 rest = length % 16;
	int i, j;

	for (i = 0; i < lines; i++) {
		for (j = 0; j < 16; j++)
			diag_printf("%02X ", address[i * 16 + j]);
		diag_printf("     ");
		for (j = 0; j < 16; j++) {
			if (isprint(address[i * 16 + j])) {
				diag_printf("%c", address[i * 16 + j]);
			}
			else {
				diag_printf(".");
			}
		}
		diag_printf("\n");
	}

	// behandle Restzeile
	for (j = 0; j < rest; j++)
		diag_printf("%02X ", address[i * 16 + j]);
	diag_printf("     ");
	for (j = 0; j < 16 - rest; j++)
		diag_printf("   ");
	for (j = 0; j < rest; j++) {
		if (isprint(address[i * 16 + j])) {
			diag_printf("%c", address[i * 16 + j]);
		}
		else {
			diag_printf(".");
		}
	}

	diag_printf("\n");

}
#endif

int get_dir_data(drive_spec *ds, COMPAR *pp)
{
	u32		actual_block;
	u32		block_needed;
	u32		entry_offset;
	int		i, j, k;
	unsigned char	*buffer;
	dir_data	dentry;

	k = 0;
	actual_block = 0;
	ds->number_of_files = 0;
	buffer = pp->dbuffer;

	for (i = 0; i < ds->maximum_root_entries; i++) {
		block_needed = ds->root_dir_start_block + i * 32 / ds->bytes_per_sector;
		if (actual_block != block_needed) {
			if (read_single_block(pp, block_needed * 512) < 0) {
				diag_printf("read single block command failed!\n");
				return -1;
			}

			actual_block = block_needed;
		}
		entry_offset = (i * 32) % ds->bytes_per_sector;
		if (		   buffer[entry_offset] == 0x00 || buffer[entry_offset] == 0x05
				|| buffer[entry_offset] == 0x2e || buffer[entry_offset] == 0xe5)
				continue;
		if (buffer[entry_offset + 0x0b] & 0x1e)
			continue;		// hidden system or label

		// copy filename
		for (j = 0;j < 8;j++)
			if (buffer[entry_offset + j] > 0x40 && buffer[entry_offset + j] < 0x5b)
				(dentry.dir_fname)[j] = buffer[entry_offset + j] + 0x20;
			else
				(dentry.dir_fname)[j] = buffer[entry_offset + j];

		for (j = 0;j < 3;j++)
			if (buffer[entry_offset + j + 8] > 0x40 && buffer[entry_offset + j + 8] < 0x5b)
				(dentry.dir_fname)[j + 9] = buffer[entry_offset + j + 8] + 0x20;
			else
				(dentry.dir_fname)[j + 9] = buffer[entry_offset + j + 8];

		dentry.dir_fname[8] = '.';
		dentry.dir_fname[12] = '\0';
		kill_space(dentry.dir_fname);

		dentry.dir_year = ((buffer[entry_offset + 0x19] & 0xfe) >> 1) + 1980;
		dentry.dir_month = ((buffer[entry_offset + 0x18] & 0xe0) >> 5) + (buffer[entry_offset + 0x19] & 0x01) * 8;
		dentry.dir_day = (buffer[entry_offset + 0x18] & 0x1f);

		dentry.dir_second = (buffer[entry_offset + 0x16] & 0x1f);
		dentry.dir_minute = ((buffer[entry_offset + 0x16] & 0xe0) >> 5) + (buffer[entry_offset + 0x17] & 0x07) * 8;
		dentry.dir_hour = (buffer[entry_offset + 0x17] & 0xf1) >> 3;

		dentry.dir_start_cluster =buffer[entry_offset + 0x1a] + ((buffer[entry_offset + 0x1b]) << 8);
		dentry.dir_fsize = ((buffer[entry_offset + 0x1c]) << 0)
								+ ((buffer[entry_offset + 0x1d]) << 8)
								+ ((buffer[entry_offset + 0x1e]) << 16)
								+ ((buffer[entry_offset + 0x1f]) << 24);

		ds->number_of_files++;
		dump_dir(&dentry, ds);
	}
	return 0;
}

void dump_dir(dir_data *dird, drive_spec *ds)
{
	int j;

	diag_printf("%2d: %s", ds->number_of_files, (dird)->dir_fname);
	for (j = 0; j < 15 - strlen((dird)->dir_fname); j++) {
		diag_printf(" ");
	}
	diag_printf("%02d.%02d.%04d    %02d:%02d:%02d    size: %10lu bytes   start cluster: %6u\n",
		(dird)->dir_day,
		(dird)->dir_month,
		(dird)->dir_year,
		(dird)->dir_hour,
		(dird)->dir_minute,
		(dird)->dir_second,
		(dird)->dir_fsize,
		(dird)->dir_start_cluster);
}

int get_next_cluster(dir_data *dird, drive_spec *ds,  COMPAR *pp, unsigned short last_cluster)
{
	u16		cluster;
	u32		block;
	static u16	fat[256];
	static u32	actual_fat_block = 0;
	u32		fat_block_needed;
	u16		fat_entry;
	u16		fvalue;

	// find next block in fat
	fat_block_needed = ds->fat_start_block + last_cluster * 2 / ds->bytes_per_sector;
	fat_entry = (last_cluster * 2) % ds->bytes_per_sector;   // this is the offset to the sector start
	if (actual_fat_block != fat_block_needed) {
#if SDCARD_DEBUG
		diag_printf("loading fat block %lu\n", fat_block_needed);
#endif
		pp->dbuffer = (unsigned char *)fat;
		if (read_single_block(pp, fat_block_needed * 512) < 0) {
			diag_printf("read single block command failed!\n");
			return -1;
		}

		actual_fat_block = fat_block_needed;
#if SDCARD_DEBUG
		dump_buf((u8 *)fat, 512);
#endif
	}

	fvalue = fat[fat_entry / 2];
#if SDCARD_DEBUG
	diag_printf("cluster is %d\n", cluster);

	diag_printf("fat contents of entry %d is: %d\n", fat_entry, fvalue);
	diag_printf("length = %lu \n", length);
#endif
	if (!fvalue) {
		diag_printf("unexpected NULL pointer in FAT\n");
		return -1;
	}
	if ((fvalue & 0xfff0) == 0xfff0) {
		//diag_printf("end of file found in FAT\n");
		return 0;
	}
	cluster = fvalue;
	return (int)cluster;

	block = (cluster - 2) * ds->sectors_per_cluster + ds->data_area_start_block;

}

int read_file(dir_data *dird, drive_spec *ds, unsigned long *address, COMPAR *pp)
{
	u32	length;
	u32	block;
	u32	*daddr;
	u16	fat[256];
	u32	actual_fat_block;
	u8	*bp;
	int	new_cluster;
	u32	clusters;
	u32	max_clusters;
	u16	start_cluster;			// first cluster of multiple block region
	int	got_eof;
#ifdef CYGBLD_BUILD_REDBOOT_WITH_WINCE_SUPPORT
	u8      blockBuffer[512];

	// Read the first block to determine if we are about
	// to load Wince .BIN image

	pp->isWinceImage = 0;
	pp->dbuffer = blockBuffer;
	block = (dird->dir_start_cluster - 2) * ds->sectors_per_cluster + ds->data_area_start_block;

	if (read_single_block(pp, block * 512) < 0) {
		diag_printf("failed to read the first data block!\n");
		return -1;
	}

	// Check signature

	if (ce_is_bin_image((void*)blockBuffer, 512)) {
		// Wince image, setup the .BIN parser.
		pp->isWinceImage = 1;
		ce_bin_init_parser();
	} else {
		pp->isWinceImage = 0;
	}
#endif
	max_clusters = 1020 / ds->sectors_per_cluster;
	got_eof = 0;

	bp = (u8 *)fat;

	actual_fat_block = 0;	//no fat block is loaded already
	daddr = address;
	start_cluster = dird->dir_start_cluster;
	length = dird->dir_fsize;
	block = (start_cluster - 2) * ds->sectors_per_cluster + ds->data_area_start_block;
	//diag_printf("Sektors per Cluster: %d\n", ds->sectors_per_cluster);

	while (length > 0) {
		u16	last_cluster = start_cluster;	// last cluster of multiple block region

		clusters = 1;

		if (length > (ds->bytes_per_sector * ds->sectors_per_cluster)) {
			length -= (ds->bytes_per_sector * ds->sectors_per_cluster);
		} else {
			length = 0;
		}

		while (clusters < max_clusters) {
			new_cluster = get_next_cluster(dird, ds,  pp, last_cluster);
			if (new_cluster < 0) {
				diag_printf("error !!!!\n");
				return -1;
			}
			if (new_cluster == 0) {
				//diag_printf("EOF number of clusters is %d !!!!\n", clusters);
				got_eof = 1;
				break;
			}
			if ((new_cluster - last_cluster) == 1) {
				last_cluster = (u16)new_cluster;
				//diag_printf("%d: cluster number %d found\n", clusters, last_cluster);
				clusters++;
				if (length > (ds->bytes_per_sector * ds->sectors_per_cluster)) {
					length -= (ds->bytes_per_sector * ds->sectors_per_cluster);
				}
				else {
					length = 0;
				}
			} else {
				break;
			}
		}
#if SDCARD_DEBUG
		diag_printf("start cluster is: %d \n", start_cluster);
		diag_printf("last cluster is:  %d \n", last_cluster);
		diag_printf("number of clusters are: %d\n", last_cluster - start_cluster + 1);
		diag_printf("length is: %d\n", length);
#endif
		if (got_eof && (length != 0)) {
			diag_printf("found unexpected EOF!\n");
			return -1;
		}

		block = (start_cluster - 2) * ds->sectors_per_cluster + ds->data_area_start_block;
		pp->dbuffer = (unsigned char *)daddr;

		if (read_multiple_block(pp, block * 512, ds->sectors_per_cluster * 
					(last_cluster - start_cluster + 1)) < 0) {
			diag_printf("read multiple block command failed!\n");
			return -1;
		}
		daddr += (ds->bytes_per_sector / 4 * ds->sectors_per_cluster *
			  (last_cluster - start_cluster + 1));

		start_cluster = get_next_cluster(dird, ds,  pp, last_cluster);
	}

	return 0;

}

int read_file_old(dir_data *dird, drive_spec *ds, unsigned long *address, COMPAR *pp)
{
	u16	cluster;
	u32	length;
	u32	block;
	u32	*daddr;
	u16	fat[256];
	u32	actual_fat_block;
	u32	fat_block_needed;
	u16	fat_entry;
	u16	fvalue;

	actual_fat_block = 0;	//no fat block is loaded already
	daddr = address;
	cluster = dird->dir_start_cluster;
	length = dird->dir_fsize;
	block = (cluster - 2) * ds->sectors_per_cluster + ds->data_area_start_block;

	diag_printf("Sektors per Cluster: %d\n", ds->sectors_per_cluster);

	while (length > 0) {
#if 0
		for (s = 0; s < ds->sectors_per_cluster; s++) {
			if (length == 0)
				break;

			pp->dbuffer = (unsigned char *)daddr;
			if (read_single_block(pp, block * 512) < 0) {
				diag_printf("read single block command failed!\n");
				return -1;
			}
#if SDCARD_DEBUG
			diag_printf("loading block: %d\n", block);
#endif
			block++;
			daddr += ds->bytes_per_sector / 4;
			if (length > ds->bytes_per_sector) {
				length -= ds->bytes_per_sector;
			} else {
				length = 0;
			}
#if SDCARD_DEBUG
			diag_printf("new length is: %lu    \n", length);
#endif
		}
#else
		pp->dbuffer = (unsigned char *)daddr;
		if (read_multiple_block(pp, block * 512, ds->sectors_per_cluster) < 0) {
			diag_printf("read multiple block command failed!\n");
			return -1;
		}

		block += ds->sectors_per_cluster;
		daddr += (ds->bytes_per_sector / 4 * ds->sectors_per_cluster);
		if (length > (ds->bytes_per_sector * ds->sectors_per_cluster)) {
			length -= (ds->bytes_per_sector * ds->sectors_per_cluster);
		} else {
			length = 0;
		}
#if SDCARD_DEBUG
		diag_printf("new length is: %lu    \n", length);
#endif
#endif
		if (!length) {			// all data loaded
			return 0;
		}

		// find next block in fat
		fat_block_needed = ds->fat_start_block + cluster * 2 / ds->bytes_per_sector;
		fat_entry = (cluster * 2) % ds->bytes_per_sector;   // this is the offset to the sector start
		if (actual_fat_block != fat_block_needed) {
#if SDCARD_DEBUG
			diag_printf("loading fat block %lu\n", fat_block_needed);
#endif
			pp->dbuffer = (unsigned char *)fat;
			if (read_single_block(pp, fat_block_needed * 512) < 0) {
				diag_printf("read single block command failed!\n");
				return -1;
			}

			actual_fat_block = fat_block_needed;
#if SDCARD_DEBUG
			dump_buf((u8 *)fat, 512);
#endif
		}

		fvalue = fat[fat_entry / 2];
#if SDCARD_DEBUG
		diag_printf("cluster is %d\n", cluster);

		diag_printf("fat contents of entry %d is: %d\n", fat_entry, fvalue);
		diag_printf("length = %lu \n", length);
#endif
		if (!fvalue) {
			diag_printf("unexpected NULL pointer in FAT\n");
			return -1;
		}
		if ((fvalue & 0xfff0) == 0xfff0) {
			diag_printf("unexpected end of file or reserved cluster found in FAT\n");
			return -1;
		}
		cluster = fvalue;
		block = (cluster - 2) * ds->sectors_per_cluster + ds->data_area_start_block;
	}

	return 0;
}

int get_dir_entry(dir_data *dentry, drive_spec *ds, COMPAR *pp, unsigned char *filename)
{
	u32		actual_block;
	u32		block_needed;
	u32		entry_offset;
	int		i, j, k;
	unsigned char	*buffer;
	unsigned char	*cp;
	int		fnlength;

	cp = filename;
	fnlength = 0;

	while (*(cp++) != '\0') {
		fnlength++;
	}
	//diag_printf("filename length is %d\n", fnlength);

	k = 0;
	actual_block = 0;
	ds->number_of_files = 0;
	buffer = pp->dbuffer;

	for (i = 0; i < ds->maximum_root_entries; i++) {
		block_needed = ds->root_dir_start_block + i * 32 / ds->bytes_per_sector;
		if (actual_block != block_needed) {
			if (read_single_block(pp, block_needed * 512) < 0) {
				diag_printf("read single block command failed!\n");
				return -1;
			}

			actual_block = block_needed;
		}
		entry_offset = (i * 32) % ds->bytes_per_sector;
		if (buffer[entry_offset] == 0x00 ||
		    buffer[entry_offset] == 0x05 ||
		    buffer[entry_offset] == 0x2e ||
		    buffer[entry_offset] == 0xe5) {
			continue;
		}
		if (buffer[entry_offset + 0x0b] & 0x1e) {
			continue;		// hidden system or label
		}
		// copy filename
		for (j = 0;j < 8;j++)
			if (buffer[entry_offset + j] > 0x40 && buffer[entry_offset + j] < 0x5b)
				(dentry->dir_fname)[j] = buffer[entry_offset + j] + 0x20;
			else
				(dentry->dir_fname)[j] = buffer[entry_offset + j];

		for (j = 0;j < 3;j++)
			if (buffer[entry_offset + j + 8] > 0x40 && buffer[entry_offset + j + 8] < 0x5b)
				(dentry->dir_fname)[j + 9] = buffer[entry_offset + j + 8] + 0x20;
			else
				(dentry->dir_fname)[j + 9] = buffer[entry_offset + j + 8];

		dentry->dir_fname[8] = '.';
		dentry->dir_fname[12] = '\0';
		kill_space(dentry->dir_fname);

		dentry->dir_year = ((buffer[entry_offset + 0x19] & 0xfe) >> 1) + 1980;
		dentry->dir_month = ((buffer[entry_offset + 0x18] & 0xe0) >> 5) + (buffer[entry_offset + 0x19] & 0x01) * 8;
		dentry->dir_day = (buffer[entry_offset + 0x18] & 0x1f);

		dentry->dir_second = (buffer[entry_offset + 0x16] & 0x1f);
		dentry->dir_minute = ((buffer[entry_offset + 0x16] & 0xe0) >> 5) + (buffer[entry_offset + 0x17] & 0x07) * 8;
		dentry->dir_hour = (buffer[entry_offset + 0x17] & 0xf1) >> 3;

		dentry->dir_start_cluster = buffer[entry_offset + 0x1a] + ((buffer[entry_offset + 0x1b]) << 8);
		dentry->dir_fsize = ((buffer[entry_offset + 0x1c]) << 0)
			+ ((buffer[entry_offset + 0x1d]) << 8)
			+ ((buffer[entry_offset + 0x1e]) << 16)
			+ ((buffer[entry_offset + 0x1f]) << 24);

		// compare filename
		cp = filename;
		for (k = 0; k < fnlength; k++) {
			if (*(cp++) != dentry->dir_fname[k]) {
				break;
			}
		}
		//diag_printf("k= %d \n", k);
		if (k == fnlength) {
#if SDCARD_DEBUG
			diag_printf("file found !!!\n");
			dump_dir(&dentry, ds);
#endif
			return 0;
		}
		ds->number_of_files++;
	}
	return -1;
}

int isprint(int character)
{
	unsigned char a;

	a = (unsigned char)(character & 0xff);

	if (a > 0x1f && a < 0x7f)
		return 1;
	else
		return 0;
}

void kill_space(char *string)
{
	int i = 0;
	int j;

	while (string[i] != '\0') {
		if (string[i] == ' ') {
			j = i;
			while (string[j] != '\0') {
				string[j] = string[j + 1];
				j++;
			}
		} else {
			i++;
		}
	}
}

int read_multiple_block(COMPAR *pp, unsigned long address, unsigned long blocks)
{
	static unsigned long __attribute__ ((aligned (32))) dma_desc[64][4];
	unsigned long val, cvalue, length, last_size, dma_buffer;
	int i;
	unsigned long	cebuffer[1024];
	int		bi;
	unsigned long	*bap;

	//HAL_WRITE_UINT32(GPSRa, 0x800);
	//HAL_WRITE_UINT32(GPCRa, 0x800);

	if (!pp->isWinceImage) {	// use chained descriptor list dma
		dma_buffer = (unsigned long)pp->dbuffer;

		length = blocks * 512;

		i = 0;
		while (length > 0) {
			if (i > 63) {
				diag_printf("descriptor overrun!!!\n");
				return -1;
			}

			dma_desc[i][0] = (unsigned long)&dma_desc[i + 1][0];
			dma_desc[i][1] = 0x41100040;	// source address is SDCARD FIFO
			dma_desc[i][2] = dma_buffer;	// target address
			val = 0x60034000;
			if (length >= 8160) {
				last_size = 8160;
				dma_desc[i][3] = val | 8160;	// command
				//diag_printf("descriptor %d: DMA length set to %d bytes\n", i, last_size);
			} else {
				last_size = length;
				dma_desc[i][3] = val | length;	// command
				//diag_printf("descriptor %d: DMA length set to %d bytes\n", i, last_size);
			}
			dma_buffer += last_size;
			length -= last_size;
			if (length == 0) {
				dma_desc[i][0] |= 1;		// set stop bit
			}
			i++;
		}

		HAL_DCACHE_SYNC();

		//diag_printf("descriptor start is 0x%08X\n", (unsigned long)&dma_desc[0][0]);

		HAL_READ_UINT32(DCSR8, val);
		val &= 0x3fffffff;			// clear RUN and NODESCRIPTOR FETCH
		HAL_WRITE_UINT32(DCSR8, val);

		HAL_WRITE_UINT32(DDADR8, (unsigned long)&dma_desc[0][0]);
		val |= 0x80000000;			// run channel
		HAL_WRITE_UINT32(DCSR8, val);

		HAL_WRITE_UINT32(MMCNOB, blocks);

		pp->command_parameter_low = address & 0xffff;
		pp->command_parameter_high = (address & 0xffff0000) >> 16;

		pp->command_number = SD_CMD18;
		pp->res_type = CMD_RES_TYPE_R1;
		pp->data_en = 1;
		pp->dma_en = 1;
		pp->data_write = 0;
		pp->stream_blk = 0;
		pp->busy = 0;
		pp->init = 0;
		pp->sd_4dat = 1;

		HAL_WRITE_UINT32(MMCCMD, pp->command_number);		// command number
		HAL_WRITE_UINT32(MMCARGH, pp->command_parameter_high);	// command parameter upper 16 bits
		HAL_WRITE_UINT32(MMCARGL, pp->command_parameter_low);	// command parameter lower 16 bits

		cvalue = cmd_value(pp);
		HAL_WRITE_UINT32(MMCCMDAT, cvalue);

		// wait for dma end
		HAL_READ_UINT32(DCSR8, val);		// wait for dma end
		while (val & 0x80000000) {
			HAL_READ_UINT32(DCSR8, val);
		}
#if SDCARD_DEBUG
		diag_printf("DMA finished \n");
		diag_printf("DMA Control Status is: 0x%08X\n", val);

		HAL_READ_UINT32(DDADR8, val);
		diag_printf("Descriptor Address Register: 0x%08X\n", val);

		HAL_READ_UINT32(DSADR8, val);
		diag_printf("Source Address Register: 0x%08X\n", val);

		HAL_READ_UINT32(DTADR8, val);
		diag_printf("Target Address Register: 0x%08X\n", val);

		HAL_READ_UINT32(DCMD8, val);
		diag_printf("Command Register: 0x%08X\n", val);
#endif
		HAL_WRITE_UINT32(MMCNOB, 1);

		pp->command_number = SD_CMD12;		//Stop Transmission
		if (send_command(pp) < 0) {
			diag_printf("CMD12 failed !!!\n");
			return -1;
		}

		return 0;
	} else {		// read wince image
#ifdef CYGBLD_BUILD_REDBOOT_WITH_WINCE_SUPPORT
		HAL_WRITE_UINT32(MMCNOB, blocks);
		//HAL_WRITE_UINT32(MMCNOB, blocks);

		pp->command_parameter_low = address & 0xffff;
		pp->command_parameter_high = (address & 0xffff0000) >> 16;

		pp->command_number = SD_CMD18;
		pp->res_type = CMD_RES_TYPE_R1;
		pp->data_en = 1;
		pp->dma_en = 0;
		pp->data_write = 0;
		pp->stream_blk = 0;
		pp->busy = 0;
		pp->init = 0;
		pp->sd_4dat = 1;

		HAL_WRITE_UINT32(MMCCMD, pp->command_number);
		HAL_WRITE_UINT32(MMCARGH, pp->command_parameter_high);
		HAL_WRITE_UINT32(MMCARGL, pp->command_parameter_low);

		cvalue = cmd_value(pp);
		HAL_WRITE_UINT32(MMCCMDAT, cvalue);

		length = 512 * blocks;

		bap = cebuffer;

		bi = 0;

		while (length > 0) {
			HAL_READ_UINT32(MMCIREG, val);
			while (!(val & 0x20)) {	// wait for RXFIFO_RD_REQ
				HAL_READ_UINT32(MMCIREG, val);
			}

			*(bap++) = *(unsigned long *)MMCRXFIFO;
			*(bap++) = *(unsigned long *)MMCRXFIFO;
			*(bap++) = *(unsigned long *)MMCRXFIFO;
			*(bap++) = *(unsigned long *)MMCRXFIFO;
			*(bap++) = *(unsigned long *)MMCRXFIFO;
			*(bap++) = *(unsigned long *)MMCRXFIFO;
			*(bap++) = *(unsigned long *)MMCRXFIFO;
			*(bap++) = *(unsigned long *)MMCRXFIFO;

			length -= 32;
			bi += 8;

			if (bi == 1024) {
				switch (ce_bin_parse_next(cebuffer, 4096)) {
				case CE_PR_EOF:   // End of file
					// The image was successfully loaded, just go out
					break;

				case CE_PR_ERROR:
					// Ops! Corrupted or invalid image!
					return -1;

				case CE_PR_MORE:
					// Need more data. Just process to the next block
					break;
				}
				bi = 0;
				bap = cebuffer;
			}

			if (!length) {
				switch (ce_bin_parse_next(cebuffer, bi * 4)) {
				case CE_PR_EOF:   // End of file
					// The image was successfully loaded, just go out
					break;
					//return 0;
				case CE_PR_ERROR:
					// Ops! Corrupted or invalid image!
					return -1;
				case CE_PR_MORE:
					// Need more data. Just process to the next block
					break;
				}
			}

		}

		if (wait4cmd_end() < 0) {
			diag_printf("read multiple block command timed out !!!!");
			return -1;
		}

		HAL_WRITE_UINT32(MMCNOB, 1);

		pp->command_number = SD_CMD12;	// Stop Transmission
		if (send_command(pp) < 0) {
			diag_printf("CMD12 failed !!!\n");
			return -1;
		}
#endif
		return 0;
	}
}
